@@ -19,7 +19,7 @@
 #include "base/i18n/rtl.h"
 #include "base/i18n/string_compare.h"
 #include "base/lazy_instance.h"
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
@@ -62,6 +62,7 @@ static const char* const kAcceptLanguageList[] = {
     "br",     // Breton
     "bs",     // Bosnian
     "ca",     // Catalan
+    "ceb",    // Cebuano
     "ckb",    // Kurdish (Arabci),  Sorani
     "co",     // Corsican
     "cs",     // Czech
@@ -119,10 +120,12 @@ static const char* const kAcceptLanguageList[] = {
     "hi",      // Hindi
     "hmn",     // Hmong
     "hr",      // Croatian
+    "ht",      // Haitian Creole
     "hu",      // Hungarian
     "hy",      // Armenian
     "ia",      // Interlingua
     "id",      // Indonesian
+    "ig",      // Igbo
     "is",      // Icelandic
     "it",      // Italian
     "it-CH",   // Italian (Switzerland)
@@ -142,6 +145,8 @@ static const char* const kAcceptLanguageList[] = {
     "lo",      // Laothian
     "lt",      // Lithuanian
     "lv",      // Latvian
+    "mg",      // Malagasy
+    "mi",      // Maori
     "mk",      // Macedonian
     "ml",      // Malayalam
     "mn",      // Mongolian
@@ -149,11 +154,13 @@ static const char* const kAcceptLanguageList[] = {
     "mr",      // Marathi
     "ms",      // Malay
     "mt",      // Maltese
+    "my",      // Burmese
     "nb",      // Norwegian (Bokmal)
     "ne",      // Nepali
     "nl",      // Dutch
     "nn",      // Norwegian (Nynorsk)
     "no",      // Norwegian
+    "ny",      // Nyanja
     "oc",      // Occitan
     "om",      // Oromo
     "or",      // Oriya
@@ -210,8 +217,10 @@ static const char* const kAcceptLanguageList[] = {
 // Returns true if |locale_name| has an alias in the ICU data file.
 bool IsDuplicateName(const std::string& locale_name) {
   static const char* const kDuplicateNames[] = {
+    "ar_001",
     "en",
     "en_001",
+    "en_150",
     "pt", // pt-BR and pt-PT are used.
     "zh",
     "zh_hans_cn",
@@ -259,15 +268,7 @@ bool IsLocaleAvailable(const std::string& locale) {
   if (!l10n_util::IsLocaleSupportedByOS(locale))
     return false;
 
-  // If the ResourceBundle is not yet initialized, return false to avoid the
-  // CHECK failure in ResourceBundle::GetSharedInstance().
-  if (!ResourceBundle::HasSharedInstance())
-    return false;
-
-  // TODO(hshi): make ResourceBundle::LocaleDataPakExists() a static function
-  // so that this can be invoked without initializing the global instance.
-  // See crbug.com/230432: CHECK failure in GetUserDataDir().
-  return ResourceBundle::GetSharedInstance().LocaleDataPakExists(locale);
+  return ui::ResourceBundle::LocaleDataPakExists(locale);
 }
 #endif
 
@@ -334,7 +335,7 @@ std::string GetLanguage(const std::string& locale) {
   return std::string(locale, 0, hyphen_pos);
 }
 
-// TOOD(jshin): revamp this function completely to use a more sytematic
+// TODO(jshin): revamp this function completely to use a more sytematic
 // and generic locale fallback based on ICU/CLDR.
 bool CheckAndResolveLocale(const std::string& locale,
                            std::string* resolved_locale) {
@@ -470,6 +471,10 @@ std::string GetApplicationLocaleInternal(const std::string& pref_locale) {
 
 #elif defined(OS_ANDROID)
 
+  // Try pref_locale first.
+  if (!pref_locale.empty())
+    candidates.push_back(base::i18n::GetCanonicalLocale(pref_locale));
+
   // On Android, query java.util.Locale for the default locale.
   candidates.push_back(base::android::GetDefaultLocaleString());
 
@@ -581,7 +586,7 @@ base::string16 GetDisplayNameForLocale(const std::string& locale,
 
     int actual_size = uloc_getDisplayName(
         locale_code.c_str(), display_locale.c_str(),
-        base::WriteInto(&display_name, kBufferSize), kBufferSize - 1, &error);
+        (UChar*)base::WriteInto(&display_name, kBufferSize), kBufferSize - 1, &error);
     DCHECK(U_SUCCESS(error));
     display_name.resize(actual_size);
   }
@@ -695,7 +700,7 @@ std::string GetStringUTF8(int message_id) {
 }
 
 base::string16 GetStringUTF16(int message_id) {
-  ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+  ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
   base::string16 str = rb.GetLocalizedString(message_id);
   AdjustParagraphDirectionality(&str);
 
@@ -709,7 +714,7 @@ base::string16 GetStringFUTF16(int message_id,
   // a StringPiece and were able to call ReplaceStringPlaceholders with
   // a StringPiece format string and base::string16 substitution strings.  In
   // practice, the strings should be relatively short.
-  ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+  ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
   const base::string16& format_string = rb.GetLocalizedString(message_id);
 
 #ifndef NDEBUG
@@ -890,6 +895,17 @@ void GetAcceptLanguagesForLocale(const std::string& display_locale,
   }
 }
 
+bool IsLanguageAccepted(const std::string& display_locale,
+                        const std::string& locale) {
+  for (const char* accept_language : kAcceptLanguageList) {
+    if (accept_language == locale &&
+        l10n_util::IsLocaleNameTranslated(locale.c_str(), display_locale)) {
+      return true;
+    }
+  }
+  return false;
+}
+
 int GetLocalizedContentsWidthInPixels(int pixel_resource_id) {
   int width = 0;
   base::StringToInt(l10n_util::GetStringUTF8(pixel_resource_id), &width);
@@ -902,7 +918,7 @@ const char* const* GetAcceptLanguageListForTesting() {
 }
 
 size_t GetAcceptLanguageListSizeForTesting() {
-  return arraysize(kAcceptLanguageList);
+  return base::size(kAcceptLanguageList);
 }
 
 }  // namespace l10n_util
