@@ -43,8 +43,6 @@ namespace {
 
 const int32_t kEventDispositionUndefined = -1;
 
-const size_t kTenSeconds = 10 * 1000 * 1000;
-
 cc::ScrollState CreateScrollStateForGesture(const WebGestureEvent& event) {
   cc::ScrollStateData scroll_state_data;
   switch (event.GetType()) {
@@ -144,7 +142,7 @@ InputHandlerProxy::InputHandlerProxy(cc::InputHandler* input_handler,
       input_handler_(input_handler),
       synchronous_input_handler_(nullptr),
       allow_root_animate_(true),
-#ifndef NDEBUG
+#if DCHECK_IS_ON()
       expect_scroll_update_end_(false),
 #endif
       gesture_scroll_on_impl_thread_(false),
@@ -156,7 +154,9 @@ InputHandlerProxy::InputHandlerProxy(cc::InputHandler* input_handler,
       has_ongoing_compositor_scroll_or_pinch_(false),
       is_first_gesture_scroll_update_(false),
       tick_clock_(base::DefaultTickClock::GetInstance()),
-      snap_fling_controller_(std::make_unique<cc::SnapFlingController>(this)) {
+      snap_fling_controller_(std::make_unique<cc::SnapFlingController>(this)),
+      compositor_touch_action_enabled_(
+          base::FeatureList::IsEnabled(features::kCompositorTouchAction)) {
   DCHECK(client);
   input_handler_->BindToClient(this);
   cc::ScrollElasticityHelper* scroll_elasticity_helper =
@@ -165,14 +165,9 @@ InputHandlerProxy::InputHandlerProxy(cc::InputHandler* input_handler,
     scroll_elasticity_controller_.reset(
         new InputScrollElasticityController(scroll_elasticity_helper));
   }
-  compositor_event_queue_ =
-      base::FeatureList::IsEnabled(features::kVsyncAlignedInputEvents)
-          ? std::make_unique<CompositorThreadEventQueue>()
-          : nullptr;
-  scroll_predictor_ =
-      base::FeatureList::IsEnabled(features::kResamplingScrollEvents)
-          ? std::make_unique<ScrollPredictor>()
-          : nullptr;
+  compositor_event_queue_ = std::make_unique<CompositorThreadEventQueue>();
+  scroll_predictor_ = std::make_unique<ScrollPredictor>(
+      base::FeatureList::IsEnabled(features::kResamplingScrollEvents));
 }
 
 InputHandlerProxy::~InputHandlerProxy() {}
@@ -201,8 +196,7 @@ void InputHandlerProxy::HandleInputEventWithLatencyInfo(
 
   // Note: Other input can race ahead of gesture input as they don't have to go
   // through the queue, but we believe it's OK to do so.
-  if (!compositor_event_queue_ ||
-      !IsGestureScrollOrPinch(event_with_callback->event().GetType())) {
+  if (!IsGestureScrollOrPinch(event_with_callback->event().GetType())) {
     DispatchSingleInputEvent(std::move(event_with_callback),
                              tick_clock_->NowTicks());
     return;
@@ -257,43 +251,13 @@ void InputHandlerProxy::HandleInputEventWithLatencyInfo(
 void InputHandlerProxy::DispatchSingleInputEvent(
     std::unique_ptr<EventWithCallback> event_with_callback,
     const base::TimeTicks now) {
-  if (compositor_event_queue_ &&
-      IsGestureScrollOrPinch(event_with_callback->event().GetType())) {
-    if (scroll_predictor_)
-      scroll_predictor_->HandleEvent(event_with_callback->original_events(),
-                                     now, event_with_callback->event_pointer());
-
-    // Report the coalesced count only for continuous events to avoid the noise
-    // from non-continuous events.
-    if (IsContinuousGestureEvent(event_with_callback->event().GetType())) {
-      UMA_HISTOGRAM_CUSTOM_COUNTS(
-          "Event.CompositorThreadEventQueue.Continuous.HeadQueueingTime",
-          (now - event_with_callback->creation_timestamp()).InMicroseconds(), 1,
-          kTenSeconds, 50);
-
-      UMA_HISTOGRAM_CUSTOM_COUNTS(
-          "Event.CompositorThreadEventQueue.Continuous.TailQueueingTime",
-          (now - event_with_callback->last_coalesced_timestamp())
-              .InMicroseconds(),
-          1, kTenSeconds, 50);
-
-      UMA_HISTOGRAM_COUNTS_1000(
-          "Event.CompositorThreadEventQueue.CoalescedCount",
-          static_cast<int>(event_with_callback->coalesced_count()));
-    } else {
-      UMA_HISTOGRAM_CUSTOM_COUNTS(
-          "Event.CompositorThreadEventQueue.NonContinuous.QueueingTime",
-          (now - event_with_callback->creation_timestamp()).InMicroseconds(), 1,
-          kTenSeconds, 50);
-    }
-  }
-
   ui::LatencyInfo monitored_latency_info = event_with_callback->latency_info();
   std::unique_ptr<cc::SwapPromiseMonitor> latency_info_swap_promise_monitor =
       input_handler_->CreateLatencyInfoSwapPromiseMonitor(
           &monitored_latency_info);
 
   current_overscroll_params_.reset();
+
   InputHandlerProxy::EventDisposition disposition =
       HandleInputEvent(event_with_callback->event());
 
@@ -321,13 +285,19 @@ void InputHandlerProxy::DispatchSingleInputEvent(
 }
 
 void InputHandlerProxy::DispatchQueuedInputEvents() {
-  if (!compositor_event_queue_)
-    return;
-
   // Calling |NowTicks()| is expensive so we only want to do it once.
   base::TimeTicks now = tick_clock_->NowTicks();
-  while (!compositor_event_queue_->empty())
-    DispatchSingleInputEvent(compositor_event_queue_->Pop(), now);
+  while (!compositor_event_queue_->empty()) {
+    std::unique_ptr<EventWithCallback> event_with_callback =
+        compositor_event_queue_->Pop();
+    if (scroll_predictor_) {
+      scroll_predictor_->ResampleScrollEvents(
+          event_with_callback->original_events(), now,
+          event_with_callback->event_pointer());
+    }
+
+    DispatchSingleInputEvent(std::move(event_with_callback), now);
+  }
 }
 
 InputHandlerProxy::EventDisposition InputHandlerProxy::HandleInputEvent(
@@ -460,6 +430,17 @@ void InputHandlerProxy::RecordMainThreadScrollingReasons(
   DCHECK(
       !cc::MainThreadScrollingReason::HasNonCompositedScrollReasons(reasons));
 
+  int32_t event_disposition_result =
+      (device == blink::kWebGestureDeviceTouchpad ? mouse_wheel_result_
+                                                  : touch_result_);
+  if (event_disposition_result == DID_NOT_HANDLE) {
+    // We should also collect main thread scrolling reasons if a scroll event
+    // scrolls on impl thread but is blocked by main thread event handlers.
+    reasons |= (device == blink::kWebGestureDeviceTouchpad
+                    ? cc::MainThreadScrollingReason::kWheelEventHandlerRegion
+                    : cc::MainThreadScrollingReason::kTouchEventHandlerRegion);
+  }
+
   // UMA_HISTOGRAM_ENUMERATION requires that the enum_max must be strictly
   // greater than the sample value. kMainThreadScrollingReasonCount doesn't
   // include the NotScrollingOnMain enum but the histograms do so adding
@@ -505,51 +486,6 @@ void InputHandlerProxy::RecordMainThreadScrollingReasons(
   }
 }
 
-void InputHandlerProxy::RecordScrollingThreadStatus(
-    blink::WebGestureDevice device,
-    uint32_t reasons) {
-  if (device != blink::kWebGestureDeviceTouchpad &&
-      device != blink::kWebGestureDeviceTouchscreen) {
-    return;
-  }
-
-  ScrollingThreadStatus scrolling_thread_status = SCROLLING_ON_MAIN;
-  if (reasons == cc::MainThreadScrollingReason::kNotScrollingOnMain) {
-    int32_t event_disposition_result =
-        (device == blink::kWebGestureDeviceTouchpad ? mouse_wheel_result_
-                                                    : touch_result_);
-    switch (event_disposition_result) {
-      case kEventDispositionUndefined:
-      case DID_NOT_HANDLE_NON_BLOCKING_DUE_TO_FLING:
-      case DID_HANDLE_NON_BLOCKING:
-      case DROP_EVENT:
-        scrolling_thread_status = SCROLLING_ON_COMPOSITOR;
-        break;
-      case DID_NOT_HANDLE:
-        scrolling_thread_status = SCROLLING_ON_COMPOSITOR_BLOCKED_ON_MAIN;
-        break;
-      default:
-        NOTREACHED();
-        scrolling_thread_status = SCROLLING_ON_COMPOSITOR;
-    }
-  }
-
-  // UMA_HISTOGRAM_ENUMERATION requires that the enum_max must be strictly
-  // greater than the sample value.
-  const uint32_t kScrolingThreadStatusEnumMax =
-      ScrollingThreadStatus::LAST_SCROLLING_THREAD_STATUS_VALUE + 1;
-
-  if (device == blink::kWebGestureDeviceTouchscreen) {
-    UMA_HISTOGRAM_ENUMERATION("Renderer4.GestureScrollingThreadStatus",
-                              scrolling_thread_status,
-                              kScrolingThreadStatusEnumMax);
-  } else {
-    UMA_HISTOGRAM_ENUMERATION("Renderer4.WheelScrollingThreadStatus",
-                              scrolling_thread_status,
-                              kScrolingThreadStatusEnumMax);
-  }
-}
-
 bool InputHandlerProxy::ShouldAnimate(bool has_precise_scroll_deltas) const {
 #if defined(OS_MACOSX)
   // Mac does not smooth scroll wheel events (crbug.com/574283).
@@ -566,6 +502,9 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleMouseWheel(
   if (wheel_event.dispatch_type == WebInputEvent::kEventNonBlocking) {
     // The first wheel event in the sequence should be cancellable.
     DCHECK(wheel_event.phase != WebMouseWheelEvent::kPhaseBegan);
+    // Noncancellable wheel events should have phase info.
+    DCHECK(wheel_event.phase != WebMouseWheelEvent::kPhaseNone ||
+           wheel_event.momentum_phase != WebMouseWheelEvent::kPhaseNone);
 
     result = static_cast<EventDisposition>(mouse_wheel_result_);
 
@@ -580,7 +519,7 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleMouseWheel(
   }
 
   blink::WebFloatPoint position_in_widget = wheel_event.PositionInWidget();
-  if (input_handler_->HasWheelEventHandlerAt(
+  if (input_handler_->HasBlockingWheelEventHandlerAt(
           gfx::Point(position_in_widget.x, position_in_widget.y))) {
     result = DID_NOT_HANDLE;
   } else {
@@ -610,7 +549,10 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureScrollBegin(
     const WebGestureEvent& gesture_event) {
   TRACE_EVENT0("input", "InputHandlerProxy::HandleGestureScrollBegin");
 
-#ifndef NDEBUG
+  if (scroll_predictor_)
+    scroll_predictor_->ResetOnGestureScrollBegin(gesture_event);
+
+#if DCHECK_IS_ON()
   expect_scroll_update_end_ = true;
 #endif
   cc::ScrollState scroll_state = CreateScrollStateForGesture(gesture_event);
@@ -634,9 +576,6 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureScrollBegin(
   RecordMainThreadScrollingReasons(gesture_event.SourceDevice(),
                                    scroll_status.main_thread_scrolling_reasons);
 
-  RecordScrollingThreadStatus(gesture_event.SourceDevice(),
-                              scroll_status.main_thread_scrolling_reasons);
-
   InputHandlerProxy::EventDisposition result = DID_NOT_HANDLE;
   scroll_sequence_ignored_ = false;
   in_inertial_scrolling_ = false;
@@ -673,7 +612,7 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureScrollBegin(
 InputHandlerProxy::EventDisposition
 InputHandlerProxy::HandleGestureScrollUpdate(
     const WebGestureEvent& gesture_event) {
-#ifndef NDEBUG
+#if DCHECK_IS_ON()
   DCHECK(expect_scroll_update_end_);
 #endif
 
@@ -724,7 +663,7 @@ InputHandlerProxy::HandleGestureScrollUpdate(
 
   if (snap_fling_controller_->HandleGestureScrollUpdate(
           GetGestureScrollUpdateInfo(gesture_event))) {
-#ifndef NDEBUG
+#if DCHECK_IS_ON()
     expect_scroll_update_end_ = false;
 #endif
     gesture_scroll_on_impl_thread_ = false;
@@ -754,7 +693,7 @@ InputHandlerProxy::HandleGestureScrollUpdate(
 InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureScrollEnd(
   const WebGestureEvent& gesture_event) {
   TRACE_EVENT0("input", "InputHandlerProxy::HandleGestureScrollEnd");
-#ifndef NDEBUG
+#if DCHECK_IS_ON()
   DCHECK(expect_scroll_update_end_);
   expect_scroll_update_end_ = false;
 #endif
@@ -813,7 +752,14 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HitTestTouchEvent(
           event_listener_type ==
           cc::InputHandler::TouchStartOrMoveEventListenerType::
               HANDLER_ON_SCROLLING_LAYER;
-      result = DID_NOT_HANDLE;
+      // A non-passive touch start / move will always set the whitelisted touch
+      // action to kTouchActionNone, and in that case we do not ack the event
+      // from the compositor.
+      if (compositor_touch_action_enabled_ && white_listed_touch_action &&
+          *white_listed_touch_action != cc::kTouchActionNone)
+        result = DID_HANDLE_NON_BLOCKING;
+      else
+        result = DID_NOT_HANDLE;
       break;
     }
   }
