@@ -4,27 +4,30 @@
 
 #include "ui/gfx/platform_font_win.h"
 
-#include <windows.h>
 #include <dwrite.h>
 #include <limits.h>
 #include <math.h>
 #include <stdint.h>
 #include <wchar.h>
+#include <windows.h>
+#include <wrl/client.h>
 
 #include <algorithm>
+#include <utility>
 
+#include "base/containers/flat_map.h"
 #include "base/debug/alias.h"
 #include "base/logging.h"
-#include "base/macros.h"
+#include "base/no_destructor.h"
+#include "base/stl_util.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_util.h"
 #include "base/strings/sys_string_conversions.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/win/scoped_comptr.h"
 #include "base/win/scoped_gdi_object.h"
 #include "base/win/scoped_hdc.h"
 #include "base/win/scoped_select_object.h"
-#include "base/win/win_util.h"
+#include "base/win/win_client_metrics.h"
 #include "third_party/skia/include/core/SkFontLCDConfig.h"
 #include "third_party/skia/include/core/SkRefCnt.h"
 #include "third_party/skia/include/core/SkTypeface.h"
@@ -35,32 +38,6 @@
 
 namespace {
 
-// Returns the minimum font size, using the minimum size callback, if set.
-int GetMinimumFontSize() {
-  int min_font_size = 0;
-  if (gfx::PlatformFontWin::get_minimum_font_size_callback)
-    min_font_size = gfx::PlatformFontWin::get_minimum_font_size_callback();
-  return min_font_size;
-}
-
-// Returns either minimum font allowed for a current locale or
-// lf_height + size_delta value.
-int AdjustFontSize(int lf_height, int size_delta) {
-  if (lf_height < 0) {
-    lf_height -= size_delta;
-  } else {
-    lf_height += size_delta;
-  }
-  const int min_font_size = GetMinimumFontSize();
-  // Make sure lf_height is not smaller than allowed min font size for current
-  // locale.
-  if (abs(lf_height) < min_font_size) {
-    return lf_height < 0 ? -min_font_size : min_font_size;
-  } else {
-    return lf_height;
-  }
-}
-
 // Sets style properties on |font_info| based on |font_style|.
 void SetLogFontStyle(int font_style, LOGFONT* font_info) {
   font_info->lfUnderline = (font_style & gfx::Font::UNDERLINE) != 0;
@@ -80,7 +57,7 @@ gfx::Font::Weight ToGfxFontWeight(int weight) {
 HRESULT FindDirectWriteFontForLOGFONT(IDWriteFactory* factory,
                                       LOGFONT* font_info,
                                       IDWriteFont** dwrite_font) {
-  base::win::ScopedComPtr<IDWriteGdiInterop> gdi_interop;
+  Microsoft::WRL::ComPtr<IDWriteGdiInterop> gdi_interop;
   HRESULT hr = factory->GetGdiInterop(gdi_interop.GetAddressOf());
   if (FAILED(hr)) {
     CHECK(false);
@@ -91,7 +68,7 @@ HRESULT FindDirectWriteFontForLOGFONT(IDWriteFactory* factory,
   if (SUCCEEDED(hr))
     return hr;
 
-  base::win::ScopedComPtr<IDWriteFontCollection> font_collection;
+  Microsoft::WRL::ComPtr<IDWriteFontCollection> font_collection;
   hr = factory->GetSystemFontCollection(font_collection.GetAddressOf());
   if (FAILED(hr)) {
     CHECK(false);
@@ -105,7 +82,7 @@ HRESULT FindDirectWriteFontForLOGFONT(IDWriteFactory* factory,
   base::win::ScopedGetDC screen_dc(NULL);
   base::win::ScopedSelectObject scoped_font(screen_dc, font.get());
 
-  base::win::ScopedComPtr<IDWriteFontFace> font_face;
+  Microsoft::WRL::ComPtr<IDWriteFontFace> font_face;
   hr = gdi_interop->CreateFontFaceFromHdc(screen_dc, font_face.GetAddressOf());
   if (FAILED(hr))
     return hr;
@@ -115,7 +92,7 @@ HRESULT FindDirectWriteFontForLOGFONT(IDWriteFactory* factory,
   if (SUCCEEDED(hr)) {
     hr = font_collection->GetFontFromFontFace(font_face.Get(), dwrite_font);
     if (SUCCEEDED(hr)) {
-      wcscpy_s(font_info->lfFaceName, arraysize(font_info->lfFaceName),
+      wcscpy_s(font_info->lfFaceName, base::size(font_info->lfFaceName),
                converted_font.lfFaceName);
     }
   }
@@ -141,7 +118,7 @@ HRESULT GetMatchingDirectWriteFont(LOGFONT* font_info,
 
   // Get a matching font from the system font collection exposed by
   // DirectWrite.
-  base::win::ScopedComPtr<IDWriteFontCollection> font_collection;
+  Microsoft::WRL::ComPtr<IDWriteFontCollection> font_collection;
   hr = factory->GetSystemFontCollection(font_collection.GetAddressOf());
   if (FAILED(hr)) {
     CHECK(false);
@@ -160,7 +137,7 @@ HRESULT GetMatchingDirectWriteFont(LOGFONT* font_info,
   //    same limitations with the face name as mentioned above.
   // 3. If step 2 fails then return the first family from the collection and
   //    use that.
-  base::win::ScopedComPtr<IDWriteFontFamily> font_family;
+  Microsoft::WRL::ComPtr<IDWriteFontFamily> font_family;
   BOOL exists = FALSE;
   uint32_t index = 0;
   hr = font_collection->FindFamilyName(font_info->lfFaceName, &index, &exists);
@@ -178,12 +155,12 @@ HRESULT GetMatchingDirectWriteFont(LOGFONT* font_info,
     }
 
     if (wcsncmp(font_info->lfFaceName, metrics.lfMessageFont.lfFaceName,
-                arraysize(font_info->lfFaceName))) {
+                base::size(font_info->lfFaceName))) {
       // First try the GDI compat route to get a matching DirectWrite font. If
       // that succeeds we are good. If not find a matching font from the font
       // collection.
-      wcscpy_s(font_info->lfFaceName, arraysize(font_info->lfFaceName),
-                metrics.lfMessageFont.lfFaceName);
+      wcscpy_s(font_info->lfFaceName, base::size(font_info->lfFaceName),
+               metrics.lfMessageFont.lfFaceName);
       hr = FindDirectWriteFontForLOGFONT(factory, font_info, dwrite_font);
       if (SUCCEEDED(hr))
         return hr;
@@ -227,7 +204,7 @@ HRESULT GetMatchingDirectWriteFont(LOGFONT* font_info,
   // TODO(ananta)
   // Remove the GetMatchingFonts and related code here once we get to a stable
   // state in canary.
-  base::win::ScopedComPtr<IDWriteFontList> matching_font_list;
+  Microsoft::WRL::ComPtr<IDWriteFontList> matching_font_list;
   hr = font_family->GetMatchingFonts(weight, stretch, style,
                                      matching_font_list.GetAddressOf());
   uint32_t matching_font_count = 0;
@@ -242,7 +219,7 @@ HRESULT GetMatchingDirectWriteFont(LOGFONT* font_info,
 
   base::string16 font_name;
   gfx::GetFamilyNameFromDirectWriteFont(*dwrite_font, &font_name);
-  wcscpy_s(font_info->lfFaceName, arraysize(font_info->lfFaceName),
+  wcscpy_s(font_info->lfFaceName, base::size(font_info->lfFaceName),
            font_name.c_str());
   return hr;
 }
@@ -251,14 +228,134 @@ HRESULT GetMatchingDirectWriteFont(LOGFONT* font_info,
 
 namespace gfx {
 
+namespace internal {
+
+class SystemFonts {
+ public:
+  SystemFonts() {
+    NONCLIENTMETRICS_XP metrics;
+    base::win::GetNonClientMetrics(&metrics);
+
+    // NOTE(dfried): When rendering Chrome, we do all of our own font scaling
+    // based on a number of factors, but what Windows reports to us has some
+    // (but not all) of these factors baked in, and not in a way that is
+    // display-consistent.
+    //
+    // For example, if your system DPI is 192 (200%) but you connect a monitor
+    // with a standard DPI (100%) then even if Chrome starts on the second
+    // monitor, we will be told the system font is 24pt instead of 12pt.
+    // Conversely, if the system DPI is set to 96 (100%) but all of our monitors
+    // are currently at 150%, Windows will still report 12pt fonts.
+    //
+    // The same is true with Text Zoom (a new accessibility feature). If zoom is
+    // set to 150%, then Windows will report a font size of 18pt. But again, we
+    // already take Text Zoom into account when rendering, so we want to account
+    // for that.
+    //
+    // Our system fonts are in DIPs, so we must always take what Windows gives
+    // us, figure out which adjustments it's making (and undo them), make our
+    // own adjustments for localization (for example, we always render Hindi 25%
+    // larger for readability), and only then can we store (and report) the
+    // system fonts.
+
+    // Factor in/out scale adjustment that fall outside what we can access here.
+    // This includes l10n adjustments and those we have to ask UWP or other COM
+    // interfaces for (since we don't have dependencies on that code from this
+    // module, and don't want to implicitly invoke COM for testing purposes if
+    // we don't have to).
+    gfx::PlatformFontWin::FontAdjustment font_adjustment;
+    if (PlatformFontWin::adjust_font_callback_) {
+      PlatformFontWin::adjust_font_callback_(&font_adjustment);
+    }
+
+    // Factor out system DPI scale that Windows will include in reported font
+    // sizes. Note that these are (sadly) system-wide and do not reflect
+    // specific displays' DPI.
+    double system_scale = GetSystemScale();
+    font_adjustment.font_scale /= system_scale;
+
+    // Grab each of the fonts from the NONCLIENTMETRICS block, adjust it
+    // appropriately, and store it in the font table.
+    AddFont(gfx::PlatformFontWin::SystemFont::kCaption, font_adjustment,
+            &metrics.lfCaptionFont);
+    AddFont(gfx::PlatformFontWin::SystemFont::kSmallCaption, font_adjustment,
+            &metrics.lfSmCaptionFont);
+    AddFont(gfx::PlatformFontWin::SystemFont::kMenu, font_adjustment,
+            &metrics.lfMenuFont);
+    AddFont(gfx::PlatformFontWin::SystemFont::kMessage, font_adjustment,
+            &metrics.lfMessageFont);
+    AddFont(gfx::PlatformFontWin::SystemFont::kStatus, font_adjustment,
+            &metrics.lfStatusFont);
+
+    is_initialized_ = true;
+  }
+
+  const gfx::Font& GetFont(gfx::PlatformFontWin::SystemFont system_font) const {
+    auto it = system_fonts_.find(system_font);
+    DCHECK(it != system_fonts_.end())
+        << "System font #" << static_cast<int>(system_font) << " not found!";
+    DCHECK(it->second.GetNativeFont())
+        << "Font for system font #" << static_cast<int>(system_font)
+        << " has invalid handle.";
+    return it->second;
+  }
+
+  static SystemFonts* Instance() {
+    static base::NoDestructor<SystemFonts> instance;
+    return instance.get();
+  }
+
+  static bool IsInitialized() { return is_initialized_; }
+
+ private:
+  void AddFont(gfx::PlatformFontWin::SystemFont system_font,
+               const gfx::PlatformFontWin::FontAdjustment& font_adjustment,
+               LOGFONT* logfont) {
+    // Make adjustments to the font as necessary.
+    PlatformFontWin::AdjustLOGFONT(font_adjustment, logfont);
+
+    // Cap at minimum font size.
+    logfont->lfHeight = PlatformFontWin::AdjustFontSize(logfont->lfHeight, 0);
+
+    // Create the Font object.
+    HFONT font = CreateFontIndirect(logfont);
+    DLOG_ASSERT(font);
+    system_fonts_.emplace(system_font, gfx::PlatformFontWin::HFontToFont(font));
+  }
+
+  // Returns the system DPI scale (standard DPI being 1.0).
+  // TODO(dfried): move dpi.[h|cc] somewhere in base/win so we can share this
+  // logic. However, note that the similar function in dpi.h is used many places
+  // it ought not to be.
+  static double GetSystemScale() {
+    constexpr double kDefaultDPI = 96.0;
+    base::win::ScopedGetDC screen_dc(nullptr);
+    return GetDeviceCaps(screen_dc, LOGPIXELSY) / kDefaultDPI;
+  }
+
+  // Use a flat map for faster lookups.
+  base::flat_map<gfx::PlatformFontWin::SystemFont, gfx::Font> system_fonts_;
+
+  static bool is_initialized_;
+
+  DISALLOW_COPY_AND_ASSIGN(SystemFonts);
+};
+
+// static
+bool SystemFonts::is_initialized_ = false;
+
+}  // namespace internal
+
 // static
 PlatformFontWin::HFontRef* PlatformFontWin::base_font_ref_;
 
 // static
-PlatformFontWin::AdjustFontCallback
-    PlatformFontWin::adjust_font_callback = nullptr;
-PlatformFontWin::GetMinimumFontSizeCallback
-    PlatformFontWin::get_minimum_font_size_callback = NULL;
+gfx::PlatformFontWin::AdjustFontCallback
+    PlatformFontWin::adjust_font_callback_ = nullptr;
+
+// static
+gfx::PlatformFontWin::GetMinimumFontSizeCallback
+    PlatformFontWin::get_minimum_font_size_callback_ = nullptr;
 
 IDWriteFactory* PlatformFontWin::direct_write_factory_ = nullptr;
 
@@ -266,12 +363,12 @@ IDWriteFactory* PlatformFontWin::direct_write_factory_ = nullptr;
 // Remove the CHECKs in this function once this stabilizes on the field.
 HRESULT GetFamilyNameFromDirectWriteFont(IDWriteFont* dwrite_font,
                                          base::string16* family_name) {
-  base::win::ScopedComPtr<IDWriteFontFamily> font_family;
+  Microsoft::WRL::ComPtr<IDWriteFontFamily> font_family;
   HRESULT hr = dwrite_font->GetFontFamily(font_family.GetAddressOf());
   if (FAILED(hr))
     CHECK(false);
 
-  base::win::ScopedComPtr<IDWriteLocalizedStrings> family_names;
+  Microsoft::WRL::ComPtr<IDWriteLocalizedStrings> family_names;
   hr = font_family->GetFamilyNames(family_names.GetAddressOf());
   if (FAILED(hr))
     CHECK(false);
@@ -280,7 +377,7 @@ HRESULT GetFamilyNameFromDirectWriteFont(IDWriteFont* dwrite_font,
   // Add support for retrieving the family for the current locale.
   wchar_t family_name_for_locale[MAX_PATH] = {0};
   hr = family_names->GetString(0, family_name_for_locale,
-                               arraysize(family_name_for_locale));
+                               base::size(family_name_for_locale));
   if (FAILED(hr))
     CHECK(false);
 
@@ -303,6 +400,57 @@ PlatformFontWin::PlatformFontWin(const std::string& font_name,
   InitWithFontNameAndSize(font_name, font_size);
 }
 
+// static
+void PlatformFontWin::SetGetMinimumFontSizeCallback(
+    GetMinimumFontSizeCallback callback) {
+  DCHECK(!internal::SystemFonts::IsInitialized());
+  get_minimum_font_size_callback_ = callback;
+}
+
+// static
+void PlatformFontWin::SetAdjustFontCallback(AdjustFontCallback callback) {
+  DCHECK(!internal::SystemFonts::IsInitialized());
+  adjust_font_callback_ = callback;
+}
+
+// static
+void PlatformFontWin::SetDirectWriteFactory(IDWriteFactory* factory) {
+  // We grab a reference on the DirectWrite factory. This reference is
+  // leaked, which is ok because skia leaks it as well.
+  factory->AddRef();
+  direct_write_factory_ = factory;
+}
+
+// static
+bool PlatformFontWin::IsDirectWriteEnabled() {
+  return direct_write_factory_ != nullptr;
+}
+
+// static
+const Font& PlatformFontWin::GetSystemFont(SystemFont system_font) {
+  return internal::SystemFonts::Instance()->GetFont(system_font);
+}
+
+// static
+Font PlatformFontWin::AdjustExistingFont(
+    NativeFont existing_font,
+    const FontAdjustment& font_adjustment) {
+  LOGFONT logfont;
+  auto result = GetObject(existing_font, sizeof(logfont), &logfont);
+  DCHECK(result);
+
+  // Make the necessary adjustments.
+  AdjustLOGFONT(font_adjustment, &logfont);
+
+  // Cap at minimum font size.
+  logfont.lfHeight = AdjustFontSize(logfont.lfHeight, 0);
+
+  // Create the Font object.
+  HFONT hfont = CreateFontIndirect(&logfont);
+  DCHECK(hfont);
+  return HFontToFont(hfont);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // PlatformFontWin, PlatformFont implementation:
 
@@ -365,7 +513,7 @@ std::string PlatformFontWin::GetLocalizedFontName() const {
   // locale, GetTextFace() returns the localized name.
   base::win::ScopedSelectObject font(memory_dc.Get(), font_ref_->hfont());
   wchar_t localized_font_name[LF_FACESIZE];
-  int length = GetTextFace(memory_dc.Get(), arraysize(localized_font_name),
+  int length = GetTextFace(memory_dc.Get(), base::size(localized_font_name),
                            &localized_font_name[0]);
   if (length <= 0)
     return GetFontName();
@@ -377,44 +525,15 @@ int PlatformFontWin::GetFontSize() const {
 }
 
 const FontRenderParams& PlatformFontWin::GetFontRenderParams() {
-  CR_DEFINE_STATIC_LOCAL(const FontRenderParams, params,
-      (gfx::GetFontRenderParams(FontRenderParamsQuery(), NULL)));
-  return params;
+  static const base::NoDestructor<FontRenderParams> params(
+      gfx::GetFontRenderParams(FontRenderParamsQuery(), nullptr));
+  return *params;
 }
 
 NativeFont PlatformFontWin::GetNativeFont() const {
   return font_ref_->hfont();
 }
 
-// static
-void PlatformFontWin::SetDirectWriteFactory(IDWriteFactory* factory) {
-  // We grab a reference on the DirectWrite factory. This reference is
-  // leaked, which is ok because skia leaks it as well.
-  factory->AddRef();
-  direct_write_factory_ = factory;
-}
-
-// static
-void PlatformFontWin::GetTextMetricsForFont(HDC hdc,
-                                            HFONT font,
-                                            TEXTMETRIC* text_metrics) {
-  base::win::ScopedSelectObject scoped_font(hdc, font);
-  GetTextMetrics(hdc, text_metrics);
-}
-
-// static
-int PlatformFontWin::GetFontSize(const LOGFONT& font_info) {
-  if (font_info.lfHeight < 0)
-    return -font_info.lfHeight;
-
-  base::win::ScopedGetDC screen_dc(NULL);
-  base::win::ScopedGDIObject<HFONT> font(CreateFontIndirect(&font_info));
-
-  TEXTMETRIC font_metrics = {0};
-  PlatformFontWin::GetTextMetricsForFont(screen_dc, font.get(), &font_metrics);
-  return font_metrics.tmAscent;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // Font, private:
 
@@ -438,20 +557,23 @@ void PlatformFontWin::InitWithFontNameAndSize(const std::string& font_name,
 }
 
 // static
-PlatformFontWin::HFontRef* PlatformFontWin::GetBaseFontRef() {
-  if (base_font_ref_ == NULL) {
-    NONCLIENTMETRICS_XP metrics;
-    base::win::GetNonClientMetrics(&metrics);
+void PlatformFontWin::GetTextMetricsForFont(HDC hdc,
+                                            HFONT font,
+                                            TEXTMETRIC* text_metrics) {
+  base::win::ScopedSelectObject scoped_font(hdc, font);
+  GetTextMetrics(hdc, text_metrics);
+}
 
-    if (adjust_font_callback)
-      adjust_font_callback(&metrics.lfMessageFont);
-    metrics.lfMessageFont.lfHeight =
-        AdjustFontSize(metrics.lfMessageFont.lfHeight, 0);
-    HFONT font = CreateFontIndirect(&metrics.lfMessageFont);
-    DLOG_ASSERT(font);
-    base_font_ref_ = PlatformFontWin::CreateHFontRef(font);
-    // base_font_ref_ is global, up the ref count so it's never deleted.
-    base_font_ref_->AddRef();
+// static
+PlatformFontWin::HFontRef* PlatformFontWin::GetBaseFontRef() {
+  if (base_font_ref_ == nullptr) {
+    // We'll delegate to our SystemFonts instance to give us the default
+    // message font.
+    PlatformFontWin* message_font =
+        static_cast<PlatformFontWin*>(internal::SystemFonts::Instance()
+                                          ->GetFont(SystemFont::kMessage)
+                                          .platform_font());
+    base_font_ref_ = message_font->font_ref_.get();
   }
   return base_font_ref_;
 }
@@ -465,7 +587,7 @@ PlatformFontWin::HFontRef* PlatformFontWin::CreateHFontRef(HFONT font) {
     GetTextMetricsForFont(screen_dc, font, &font_metrics);
   }
 
-  if (direct_write_factory_)
+  if (IsDirectWriteEnabled())
     return CreateHFontRefFromSkia(font, font_metrics);
 
   return CreateHFontRefFromGDI(font, font_metrics);
@@ -523,7 +645,7 @@ PlatformFontWin::HFontRef* PlatformFontWin::CreateHFontRefFromSkia(
   // Fix SkScalerContext_win_dw.cpp to return all metrics we need from
   // DirectWrite and remove the code here which retrieves metrics from
   // DirectWrite to calculate the cap height.
-  base::win::ScopedComPtr<IDWriteFont> dwrite_font;
+  Microsoft::WRL::ComPtr<IDWriteFont> dwrite_font;
   HRESULT hr = GetMatchingDirectWriteFont(
       &font_info, italic, direct_write_factory_, dwrite_font.GetAddressOf());
   if (FAILED(hr)) {
@@ -551,21 +673,20 @@ PlatformFontWin::HFontRef* PlatformFontWin::CreateHFontRefFromSkia(
       FontRenderParams::SubpixelRenderingToSkiaLCDOrientation(
           font_params.subpixel_rendering));
 
-  SkPaint paint;
-  paint.setAntiAlias(font_params.antialiasing);
-  paint.setTypeface(std::move(skia_face));
-  paint.setTextSize(-font_info.lfHeight);
-  SkPaint::FontMetrics skia_metrics;
-  paint.getFontMetrics(&skia_metrics);
+  SkFont font(std::move(skia_face), -font_info.lfHeight);
+  font.setEdging(font_params.antialiasing ? SkFont::Edging::kAntiAlias
+                                          : SkFont::Edging::kAlias);
+  SkFontMetrics skia_metrics;
+  font.getMetrics(&skia_metrics);
 
   // The calculations below are similar to those in the CreateHFontRef
   // function. The height, baseline and cap height are rounded up to ensure
   // that they match up closely with GDI.
   const int height = std::ceil(skia_metrics.fDescent - skia_metrics.fAscent);
   const int baseline = std::max<int>(1, std::ceil(-skia_metrics.fAscent));
-  const int cap_height = std::ceil(paint.getTextSize() *
-      static_cast<double>(dwrite_font_metrics.capHeight) /
-          dwrite_font_metrics.designUnitsPerEm);
+  const int cap_height = std::ceil(
+      font.getSize() * static_cast<double>(dwrite_font_metrics.capHeight) /
+      dwrite_font_metrics.designUnitsPerEm);
 
   // The metrics retrieved from skia don't have the average character width. In
   // any case if we get the average character width from skia then use that or
@@ -591,6 +712,50 @@ PlatformFontWin::HFontRef* PlatformFontWin::CreateHFontRefFromSkia(
                       ToGfxFontWeight(font_info.lfWeight), style);
 }
 
+// static
+int PlatformFontWin::AdjustFontSize(int lf_height, int size_delta) {
+  // Extract out the sign of |lf_height| - we'll add it back later.
+  const int lf_sign = lf_height < 0 ? -1 : 1;
+  lf_height = std::abs(lf_height);
+
+  // Apply the size adjustment.
+  lf_height += size_delta;
+
+  // Make sure |lf_height| is not smaller than allowed min allowed font size.
+  int min_font_size = 0;
+  if (get_minimum_font_size_callback_) {
+    min_font_size = get_minimum_font_size_callback_();
+    DCHECK_GE(min_font_size, 0);
+  }
+  lf_height = std::max(min_font_size, lf_height);
+
+  // Add back the sign.
+  return lf_sign * lf_height;
+}
+
+// static
+void PlatformFontWin::AdjustLOGFONT(
+    const gfx::PlatformFontWin::FontAdjustment& font_adjustment,
+    LOGFONT* logfont) {
+  DCHECK_GT(font_adjustment.font_scale, 0.0);
+  LONG new_height =
+      LONG(std::round(logfont->lfHeight * font_adjustment.font_scale));
+  if (logfont->lfHeight && !new_height)
+    new_height = logfont->lfHeight > 0 ? 1 : -1;
+  logfont->lfHeight = new_height;
+  if (!font_adjustment.font_family_override.empty()) {
+    auto result = wcscpy_s(logfont->lfFaceName,
+                           font_adjustment.font_family_override.c_str());
+    DCHECK_EQ(0, result) << "Font name " << font_adjustment.font_family_override
+                         << " cannot be copied into LOGFONT structure.";
+  }
+}
+
+// static
+Font PlatformFontWin::HFontToFont(HFONT hfont) {
+  return Font(new PlatformFontWin(CreateHFontRef(hfont)));
+}
+
 PlatformFontWin::PlatformFontWin(HFontRef* hfont_ref) : font_ref_(hfont_ref) {
 }
 
