@@ -14,6 +14,7 @@
 #include "base/memory/ptr_util.h"
 #include "base/strings/string_util.h"
 #include "base/threading/thread_local.h"
+#include "build/build_config.h"
 #include "ui/gl/gl_bindings.h"
 #include "ui/gl/gl_gl_api_implementation.h"
 #include "ui/gl/gl_implementation.h"
@@ -187,7 +188,17 @@ void GLContext::ForceReleaseVirtuallyCurrent() {
   NOTREACHED();
 }
 
+void GLContext::DirtyVirtualContextState() {
+  current_virtual_context_ = nullptr;
+}
+
 #if defined(OS_MACOSX)
+uint64_t GLContext::BackpressureFenceCreate() {
+  return 0;
+}
+
+void GLContext::BackpressureFenceWait(uint64_t fence) {}
+
 void GLContext::FlushForDriverCrashWorkaround() {}
 #endif
 
@@ -218,7 +229,6 @@ bool GLContext::LosesAllContextsOnContextLost() {
     case kGLImplementationEGLGLES2:
     case kGLImplementationSwiftShaderGL:
       return true;
-    case kGLImplementationOSMesaGL:
     case kGLImplementationAppleGL:
       return false;
     case kGLImplementationMockGL:
@@ -279,6 +289,7 @@ bool GLContext::WasAllocatedUsingRobustnessExtension() {
 
 void GLContext::InitializeDynamicBindings() {
   DCHECK(IsCurrent(nullptr));
+  BindGLApi();
   DCHECK(static_bindings_initialized_);
   if (!dynamic_bindings_initialized_) {
     if (real_gl_api_) {
