@@ -8,15 +8,16 @@
 #include <OpenGL/CGLTypes.h>
 
 #include <memory>
+#include <sstream>
 #include <vector>
 
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/message_loop/message_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/trace_event/trace_event.h"
 #include "ui/gl/gl_bindings.h"
+#include "ui/gl/gl_fence.h"
 #include "ui/gl/gl_gl_api_implementation.h"
 #include "ui/gl/gl_implementation.h"
 #include "ui/gl/gl_surface.h"
@@ -30,18 +31,11 @@ namespace {
 
 bool g_support_renderer_switching;
 
-struct CGLRendererInfoObjDeleter {
-  void operator()(CGLRendererInfoObj* x) {
-    if (x)
-      CGLDestroyRendererInfo(*x);
-  }
-};
-
 }  // namespace
 
-static CGLPixelFormatObj GetPixelFormat() {
+static CGLPixelFormatObj GetPixelFormat(const int core_profile = 0) {
   static CGLPixelFormatObj format;
-  if (format)
+  if (format && core_profile == 0)
     return format;
   std::vector<CGLPixelFormatAttribute> attribs;
   // If the system supports dual gpus then allow offline renderers for every
@@ -59,9 +53,8 @@ static CGLPixelFormatObj GetPixelFormat() {
     // These constants don't exist in the 10.6 SDK against which
     // Chromium currently compiles.
     const int kOpenGLProfile = 99;
-    const int kOpenGL3_2Core = 0x3200;
     attribs.push_back(static_cast<CGLPixelFormatAttribute>(kOpenGLProfile));
-    attribs.push_back(static_cast<CGLPixelFormatAttribute>(kOpenGL3_2Core));
+    attribs.push_back(static_cast<CGLPixelFormatAttribute>(core_profile));
   }
 
   attribs.push_back((CGLPixelFormatAttribute) 0);
@@ -81,15 +74,9 @@ static CGLPixelFormatObj GetPixelFormat() {
   return format;
 }
 
-GLContextCGL::GLContextCGL(GLShareGroup* share_group)
-  : GLContextReal(share_group),
-    context_(nullptr),
-    gpu_preference_(PreferIntegratedGpu),
-    discrete_pixelformat_(nullptr),
-    screen_(-1),
-    renderer_id_(-1),
-    safe_to_force_gpu_switch_(true) {
-}
+GLContextCGL::GLContextCGL(GLShareGroup* share_group, int core_profile_number)
+    : GLContextReal(share_group)
+    , core_profile_number_(core_profile_number) {}
 
 bool GLContextCGL::Initialize(GLSurface* compatible_surface,
                               const GLContextAttribs& attribs) {
@@ -106,7 +93,7 @@ bool GLContextCGL::Initialize(GLSurface* compatible_surface,
   GLContextCGL* share_context = share_group() ?
       static_cast<GLContextCGL*>(share_group()->GetContext()) : nullptr;
 
-  CGLPixelFormatObj format = GetPixelFormat();
+  CGLPixelFormatObj format = GetPixelFormat(core_profile_number_);
   if (!format)
     return false;
 
@@ -148,9 +135,9 @@ bool GLContextCGL::Initialize(GLSurface* compatible_surface,
 }
 
 void GLContextCGL::Destroy() {
-  if (!yuv_to_rgb_converters_.empty()) {
+  if (!yuv_to_rgb_converters_.empty() || !backpressure_fences_.empty()) {
     // If this context is not current, bind this context's API so that the YUV
-    // converter can safely destruct
+    // converter and GLFences can safely destruct
     GLContext* current_context = GetRealCurrent();
     if (current_context != this) {
       SetCurrentGL(GetCurrentGL());
@@ -159,6 +146,7 @@ void GLContextCGL::Destroy() {
     ScopedCGLSetCurrentContext scoped_set_current(
         static_cast<CGLContextObj>(context_));
     yuv_to_rgb_converters_.clear();
+    backpressure_fences_.clear();
 
     // Rebind the current context's API if needed.
     if (current_context && current_context != this) {
@@ -166,7 +154,7 @@ void GLContextCGL::Destroy() {
     }
   }
   if (discrete_pixelformat_) {
-    if (base::MessageLoop::current() != nullptr) {
+    if (base::ThreadTaskRunnerHandle::IsSet()) {
       // Delay releasing the pixel format for 10 seconds to reduce the number of
       // unnecessary GPU switches.
       base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
@@ -219,6 +207,7 @@ bool GLContextCGL::ForceGpuSwitchIfNeeded() {
         }
       }
       renderer_id_ = renderer_id;
+      has_switched_gpus_ = true;
     }
   }
   return true;
@@ -234,8 +223,70 @@ YUVToRGBConverter* GLContextCGL::GetYUVToRGBConverter(
   return yuv_to_rgb_converter.get();
 }
 
+constexpr uint64_t kInvalidFenceId = 0;
+
+uint64_t GLContextCGL::BackpressureFenceCreate() {
+  TRACE_EVENT0("gpu", "GLContextCGL::BackpressureFenceCreate");
+
+  // This flush will trigger a crash if FlushForDriverCrashWorkaround is not
+  // called sufficiently frequently.
+  glFlush();
+
+  if (gl::GLFence::IsSupported()) {
+    next_backpressure_fence_ += 1;
+    backpressure_fences_[next_backpressure_fence_] = GLFence::Create();
+    return next_backpressure_fence_;
+  }
+  glFinish();
+  return kInvalidFenceId;
+}
+
+void GLContextCGL::BackpressureFenceWait(uint64_t fence_id) {
+  TRACE_EVENT0("gpu", "GLContextCGL::BackpressureFenceWait");
+  if (fence_id == kInvalidFenceId) {
+    return;
+  }
+
+  // If a fence is not found, then it has already been waited on.
+  auto found = backpressure_fences_.find(fence_id);
+  if (found == backpressure_fences_.end())
+    return;
+  std::unique_ptr<GLFence> fence = std::move(found->second);
+  backpressure_fences_.erase(found);
+
+  // While we could call GLFence::ClientWait, this performs a busy wait on
+  // Mac, leading to high CPU usage. Instead we poll with a 1ms delay. This
+  // should have minimal impact, as we will only hit this path when we are
+  // more than one frame (16ms) behind.
+  //
+  // Note that on some platforms (10.9), fences appear to sometimes get
+  // lost and will never pass. Add a 32ms timout to prevent these
+  // situations from causing a GPU process hang.
+  // https://crbug.com/618075
+  bool fence_completed = false;
+  for (int poll_iter = 0; !fence_completed && poll_iter < 32; ++poll_iter) {
+    if (poll_iter > 0) {
+      base::PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(1));
+    }
+    {
+      TRACE_EVENT0("gpu", "GLFence::HasCompleted");
+      fence_completed = fence->HasCompleted();
+    }
+  }
+  if (!fence_completed) {
+    TRACE_EVENT0("gpu", "Finish");
+    // We timed out waiting for the above fence, just issue a glFinish.
+    glFinish();
+  }
+  fence.reset();
+
+  // Waiting on |fence_id| has implicitly waited on all previous fences, so
+  // remove them.
+  while (backpressure_fences_.begin()->first < fence_id)
+    backpressure_fences_.erase(backpressure_fences_.begin());
+}
+
 void GLContextCGL::FlushForDriverCrashWorkaround() {
-  TRACE_EVENT0("gpu", "GLContextCGL::FlushForDriverCrashWorkaround");
   if (!context_ || CGLGetCurrentContext() != context_)
     return;
   glFlush();
