@@ -6,10 +6,12 @@
 
 #include "base/feature_list.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "ui/accessibility/ax_enums.mojom.h"
 #include "ui/accessibility/ax_node_data.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/models/simple_menu_model.h"
+#include "ui/compositor/paint_recorder.h"
 #include "ui/compositor/scoped_layer_animation_settings.h"
 #include "ui/gfx/canvas.h"
 #include "ui/gfx/image/image_skia_operations.h"
@@ -29,13 +31,15 @@
 #include "ui/views/focus/focus_manager.h"
 #include "ui/views/widget/widget.h"
 
+#if defined(OS_WIN)
+#include "ui/base/win/shell.h"
+#endif
+
 namespace message_center {
 
 namespace {
 
-const SkColor kBorderColor = SkColorSetARGB(0x1F, 0x0, 0x0, 0x0);
-const int kShadowCornerRadius = 0;
-const int kShadowElevation = 2;
+constexpr SkColor kBorderColor = SkColorSetARGB(0x1F, 0x0, 0x0, 0x0);
 
 // Creates a text for spoken feedback from the data contained in the
 // notification.
@@ -55,8 +59,12 @@ base::string16 CreateAccessibleName(const Notification& notification) {
   return base::JoinString(accessible_lines, base::ASCIIToUTF16("\n"));
 }
 
-bool ShouldRoundMessageViewCorners() {
-  return base::FeatureList::IsEnabled(message_center::kNewStyleNotifications);
+bool ShouldShowAeroShadowBorder() {
+#if defined(OS_WIN)
+  return ui::win::IsAeroGlassEnabled();
+#else
+  return false;
+#endif
 }
 
 }  // namespace
@@ -72,18 +80,27 @@ MessageView::MessageView(const Notification& notification)
   SetPaintToLayer();
   layer()->SetFillsBoundsOpaquely(false);
 
-  // Create the opaque background that's above the view's shadow.
-  background_view_ = new views::View();
-  UpdateCornerRadius(0, 0);
-  AddChildView(background_view_);
-
   focus_painter_ = views::Painter::CreateSolidFocusPainter(
       kFocusBorderColor, gfx::Insets(0, 0, 1, 1));
 
   UpdateWithNotification(notification);
+
+  UpdateCornerRadius(0, 0);
+
+  // If Aero is enabled, set shadow border.
+  if (ShouldShowAeroShadowBorder()) {
+    const auto& shadow = gfx::ShadowDetails::Get(2, 0);
+    gfx::Insets ninebox_insets = gfx::ShadowValue::GetBlurRegion(shadow.values);
+    SetBorder(views::CreateBorderPainter(
+        views::Painter::CreateImagePainter(shadow.ninebox_image,
+                                           ninebox_insets),
+        -gfx::ShadowValue::GetMargin(shadow.values)));
+  }
 }
 
-MessageView::~MessageView() {}
+MessageView::~MessageView() {
+  RemovedFromWidget();
+}
 
 void MessageView::UpdateWithNotification(const Notification& notification) {
   pinned_ = notification.pinned();
@@ -101,36 +118,16 @@ void MessageView::SetIsNested() {
   is_nested_ = true;
   // Update enability since it might be changed by "is_nested" flag.
   slide_out_controller_.set_slide_mode(CalculateSlideMode());
+  slide_out_controller_.set_update_opacity(false);
 
-  if (ShouldRoundMessageViewCorners()) {
-    SetBorder(views::CreateRoundedRectBorder(
-        kNotificationBorderThickness, kNotificationCornerRadius, kBorderColor));
-  } else {
-    const auto& shadow =
-        gfx::ShadowDetails::Get(kShadowElevation, kShadowCornerRadius);
-    gfx::Insets ninebox_insets =
-        gfx::ShadowValue::GetBlurRegion(shadow.values) +
-        gfx::Insets(kShadowCornerRadius);
-    SetBorder(views::CreateBorderPainter(
-        views::Painter::CreateImagePainter(shadow.ninebox_image,
-                                           ninebox_insets),
-        -gfx::ShadowValue::GetMargin(shadow.values)));
-  }
+  SetBorder(views::CreateRoundedRectBorder(
+      kNotificationBorderThickness, kNotificationCornerRadius, kBorderColor));
+  if (GetControlButtonsView())
+    GetControlButtonsView()->ShowCloseButton(GetMode() != Mode::PINNED);
 }
 
-bool MessageView::IsCloseButtonFocused() const {
-  auto* control_buttons_view = GetControlButtonsView();
-  return control_buttons_view ? control_buttons_view->IsCloseButtonFocused()
-                              : false;
-}
-
-void MessageView::RequestFocusOnCloseButton() {
-  auto* control_buttons_view = GetControlButtonsView();
-  if (!control_buttons_view)
-    return;
-
-  control_buttons_view->RequestFocusOnCloseButton();
-  UpdateControlButtonsVisibility();
+void MessageView::CloseSwipeControl() {
+  slide_out_controller_.CloseSwipeControl();
 }
 
 void MessageView::SetExpanded(bool expanded) {
@@ -157,7 +154,7 @@ void MessageView::SetManuallyExpandedOrCollapsed(bool value) {
 }
 
 void MessageView::UpdateCornerRadius(int top_radius, int bottom_radius) {
-  background_view_->SetBackground(views::CreateBackgroundFromPainter(
+  SetBackground(views::CreateBackgroundFromPainter(
       std::make_unique<NotificationBackgroundPainter>(top_radius,
                                                       bottom_radius)));
   SchedulePaint();
@@ -222,9 +219,23 @@ bool MessageView::OnKeyReleased(const ui::KeyEvent& event) {
   return true;
 }
 
+void MessageView::PaintChildren(const views::PaintInfo& paint_info) {
+  views::View::PaintChildren(paint_info);
+
+  // Paint focus ring on top of all the children.
+  ui::PaintRecorder recorder(paint_info.context(), size());
+  views::Painter::PaintFocusPainter(this, recorder.canvas(),
+                                    focus_painter_.get());
+}
+
 void MessageView::OnPaint(gfx::Canvas* canvas) {
-  views::View::OnPaint(canvas);
-  views::Painter::PaintFocusPainter(this, canvas, focus_painter_.get());
+  if (ShouldShowAeroShadowBorder()) {
+    // If the border is shadow, paint border first.
+    OnPaintBorder(canvas);
+    OnPaintBackground(canvas);
+  } else {
+    views::View::OnPaint(canvas);
+  }
 }
 
 void MessageView::OnFocus() {
@@ -239,25 +250,6 @@ void MessageView::OnBlur() {
   SchedulePaint();
 }
 
-void MessageView::Layout() {
-  views::View::Layout();
-
-  gfx::Rect content_bounds = GetContentsBounds();
-
-  // Background.
-  background_view_->SetBoundsRect(content_bounds);
-
-  // ChromeOS rounds the corners of the message view. TODO(estade): should we do
-  // this for all platforms?
-  if (ShouldRoundMessageViewCorners()) {
-    gfx::Path path;
-    constexpr SkScalar kCornerRadius = SkIntToScalar(kNotificationCornerRadius);
-    path.addRoundRect(gfx::RectToSkRect(background_view_->GetLocalBounds()),
-                      kCornerRadius, kCornerRadius);
-    background_view_->set_clip_path(path);
-  }
-}
-
 const char* MessageView::GetClassName() const {
   return kViewClassName;
 }
@@ -292,27 +284,58 @@ void MessageView::OnGestureEvent(ui::GestureEvent* event) {
   event->SetHandled();
 }
 
+void MessageView::RemovedFromWidget() {
+  if (!focus_manager_)
+    return;
+  focus_manager_->RemoveFocusChangeListener(this);
+  focus_manager_ = nullptr;
+}
+
+void MessageView::AddedToWidget() {
+  focus_manager_ = GetFocusManager();
+  if (focus_manager_)
+    focus_manager_->AddFocusChangeListener(this);
+}
+
 ui::Layer* MessageView::GetSlideOutLayer() {
   return is_nested_ ? layer() : GetWidget()->GetLayer();
 }
 
-void MessageView::OnSlideChanged() {}
+void MessageView::OnSlideStarted() {
+  for (auto* observer : slide_observers_) {
+    observer->OnSlideStarted(notification_id_);
+  }
+}
+
+void MessageView::OnSlideChanged(bool in_progress) {
+  for (auto* observer : slide_observers_) {
+    observer->OnSlideChanged(notification_id_);
+  }
+}
+
+void MessageView::AddSlideObserver(MessageView::SlideObserver* observer) {
+  slide_observers_.push_back(observer);
+}
 
 void MessageView::OnSlideOut() {
-  // As a workaround for a MessagePopupCollection bug https://crbug.com/805208,
-  // pass false to by_user although it is triggered by user.
-  // TODO(tetsui): Rewrite MessagePopupCollection and remove this hack.
-  if (pinned_) {
-    // Also a workaround to not break notification pinning.
-    MessageCenter::Get()->MarkSinglePopupAsShown(
-        notification_id_, true /* mark_notification_as_read */);
-  } else {
-    MessageCenter::Get()->RemoveNotification(notification_id_,
-                                             true /* by_user */);
+  MessageCenter::Get()->RemoveNotification(notification_id_,
+                                           true /* by_user */);
+}
+
+void MessageView::OnWillChangeFocus(views::View* before, views::View* now) {}
+
+void MessageView::OnDidChangeFocus(views::View* before, views::View* now) {
+  if (Contains(before) || Contains(now) ||
+      (GetControlButtonsView() && (GetControlButtonsView()->Contains(before) ||
+                                   GetControlButtonsView()->Contains(now)))) {
+    UpdateControlButtonsVisibility();
   }
 }
 
 SlideOutController::SlideMode MessageView::CalculateSlideMode() const {
+  if (disable_slide_)
+    return SlideOutController::SlideMode::NO_SLIDE;
+
   switch (GetMode()) {
     case Mode::SETTING:
       return SlideOutController::SlideMode::NO_SLIDE;
@@ -338,9 +361,23 @@ MessageView::Mode MessageView::GetMode() const {
   return Mode::NORMAL;
 }
 
+float MessageView::GetSlideAmount() const {
+  return slide_out_controller_.gesture_amount();
+}
+
 void MessageView::SetSettingMode(bool setting_mode) {
   setting_mode_ = setting_mode;
   slide_out_controller_.set_slide_mode(CalculateSlideMode());
+  UpdateControlButtonsVisibility();
+}
+
+void MessageView::DisableSlideForcibly(bool disable) {
+  disable_slide_ = disable;
+  slide_out_controller_.set_slide_mode(CalculateSlideMode());
+}
+
+void MessageView::SetSlideButtonWidth(int control_button_width) {
+  slide_out_controller_.SetSwipeControlWidth(control_button_width);
 }
 
 void MessageView::OnCloseButtonPressed() {
@@ -356,9 +393,29 @@ void MessageView::OnSnoozeButtonPressed(const ui::Event& event) {
   // No default implementation for snooze.
 }
 
+bool MessageView::ShouldShowControlButtons() const {
+#if defined(OS_CHROMEOS)
+  // Users on ChromeOS are used to the Settings and Close buttons not being
+  // visible at all times, but users on other platforms expect them to be
+  // visible.
+  auto* control_buttons_view = GetControlButtonsView();
+  return control_buttons_view &&
+         (control_buttons_view->IsAnyButtonFocused() ||
+          (GetMode() != Mode::SETTING && IsMouseHovered()));
+#else
+  return true;
+#endif
+}
+
+void MessageView::UpdateControlButtonsVisibility() {
+  auto* control_buttons_view = GetControlButtonsView();
+  if (control_buttons_view)
+    control_buttons_view->ShowButtons(ShouldShowControlButtons());
+}
+
 void MessageView::SetDrawBackgroundAsActive(bool active) {
-  background_view_->background()->SetNativeControlColor(
-      active ? kHoveredButtonBackgroundColor : kNotificationBackgroundColor);
+  background()->SetNativeControlColor(active ? kHoveredButtonBackgroundColor
+                                             : kNotificationBackgroundColor);
   SchedulePaint();
 }
 
