@@ -27,14 +27,19 @@ struct wl_subcompositor;
 struct wl_subsurface;
 struct wl_surface;
 struct wl_touch;
+struct wp_presentation;
+struct wp_presentation_feedback;
 struct xdg_shell;
 struct xdg_surface;
 struct xdg_popup;
+struct zwp_linux_dmabuf_v1;
 struct zxdg_shell_v6;
 struct zxdg_surface_v6;
 struct zxdg_toplevel_v6;
 struct zxdg_popup_v6;
 struct zxdg_positioner_v6;
+struct zwp_text_input_manager_v1;
+struct zwp_text_input_v1;
 
 namespace wl {
 
@@ -155,6 +160,18 @@ struct ObjectTraits<wl_touch> {
   static void (*deleter)(wl_touch*);
 };
 
+template <>
+struct ObjectTraits<wp_presentation> {
+  static const wl_interface* interface;
+  static void (*deleter)(wp_presentation*);
+};
+
+template <>
+struct ObjectTraits<wp_presentation_feedback> {
+  static const wl_interface* interface;
+  static void (*deleter)(wp_presentation_feedback*);
+};
+
 template <>
 struct ObjectTraits<xdg_shell> {
   static const wl_interface* interface;
@@ -173,6 +190,12 @@ struct ObjectTraits<xdg_popup> {
   static void (*deleter)(xdg_popup*);
 };
 
+template <>
+struct ObjectTraits<zwp_linux_dmabuf_v1> {
+  static const wl_interface* interface;
+  static void (*deleter)(zwp_linux_dmabuf_v1*);
+};
+
 template <>
 struct ObjectTraits<zxdg_shell_v6> {
   static const wl_interface* interface;
@@ -203,6 +226,18 @@ struct ObjectTraits<zxdg_positioner_v6> {
   static void (*deleter)(zxdg_positioner_v6*);
 };
 
+template <>
+struct ObjectTraits<zwp_text_input_manager_v1> {
+  static const wl_interface* interface;
+  static void (*deleter)(zwp_text_input_manager_v1*);
+};
+
+template <>
+struct ObjectTraits<zwp_text_input_v1> {
+  static const wl_interface* interface;
+  static void (*deleter)(zwp_text_input_v1*);
+};
+
 struct Deleter {
   template <typename T>
   void operator()(T* obj) {
@@ -216,7 +251,7 @@ class Object : public std::unique_ptr<T, Deleter> {
   Object() {}
   explicit Object(T* obj) : std::unique_ptr<T, Deleter>(obj) {}
 
-  uint32_t id() {
+  uint32_t id() const {
     return wl_proxy_get_id(
         reinterpret_cast<wl_proxy*>(std::unique_ptr<T, Deleter>::get()));
   }
