@@ -81,7 +81,8 @@ void MenuRunnerImpl::RunMenuAt(Widget* parent,
                                MenuButton* button,
                                const gfx::Rect& bounds,
                                MenuAnchorPosition anchor,
-                               int32_t run_types) {
+                               int32_t run_types,
+                               base::flat_set<int> alerted_commands) {
   closing_event_time_ = base::TimeTicks();
   if (running_) {
     // Ignore requests to show the menu while it's already showing. MenuItemView
@@ -137,7 +138,7 @@ void MenuRunnerImpl::RunMenuAt(Widget* parent,
 
   controller->Run(parent, button, menu_, bounds, anchor,
                   (run_types & MenuRunner::CONTEXT_MENU) != 0,
-                  (run_types & MenuRunner::NESTED_DRAG) != 0);
+                  (run_types & MenuRunner::NESTED_DRAG) != 0, alerted_commands);
 }
 
 void MenuRunnerImpl::Cancel() {
@@ -192,8 +193,7 @@ void MenuRunnerImpl::SiblingMenuCreated(MenuItemView* menu) {
 
 MenuRunnerImpl::~MenuRunnerImpl() {
   delete menu_;
-  for (std::set<MenuItemView*>::iterator i = sibling_menus_.begin();
-       i != sibling_menus_.end(); ++i)
+  for (auto i = sibling_menus_.begin(); i != sibling_menus_.end(); ++i)
     delete *i;
 }
 
