@@ -7,16 +7,15 @@
 #include <algorithm>
 
 #include "build/build_config.h"
-#include "ui/base/material_design/material_design_controller.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "ui/views/background.h"
 #include "ui/views/border.h"
-#include "ui/views/controls/button/blue_button.h"
 #include "ui/views/controls/button/button.h"
 #include "ui/views/controls/button/checkbox.h"
 #include "ui/views/controls/button/image_button.h"
 #include "ui/views/controls/button/label_button.h"
 #include "ui/views/controls/button/md_text_button.h"
+#include "ui/views/event_utils.h"
 #include "ui/views/layout/grid_layout.h"
 #include "ui/views/layout/layout_provider.h"
 #include "ui/views/style/platform_style.h"
@@ -92,7 +91,8 @@ void DialogClientView::AcceptWindow() {
   // Only notify the delegate once. See |delegate_allowed_close_|'s comment.
   if (!delegate_allowed_close_ && GetDialogDelegate()->Accept()) {
     delegate_allowed_close_ = true;
-    GetWidget()->Close();
+    GetWidget()->CloseWithReason(
+        views::Widget::ClosedReason::kAcceptButtonClicked);
   }
 }
 
@@ -100,7 +100,8 @@ void DialogClientView::CancelWindow() {
   // Only notify the delegate once. See |delegate_allowed_close_|'s comment.
   if (!delegate_allowed_close_ && GetDialogDelegate()->Cancel()) {
     delegate_allowed_close_ = true;
-    GetWidget()->Close();
+    GetWidget()->CloseWithReason(
+        views::Widget::ClosedReason::kCancelButtonClicked);
   }
 }
 
@@ -164,6 +165,13 @@ gfx::Size DialogClientView::GetMaximumSize() const {
   return max_size;
 }
 
+void DialogClientView::VisibilityChanged(View* starting_from, bool is_visible) {
+  ClientView::VisibilityChanged(starting_from, is_visible);
+
+  if (is_visible)
+    view_shown_time_stamp_ = base::TimeTicks::Now();
+}
+
 void DialogClientView::Layout() {
   button_row_container_->SetSize(
       gfx::Size(width(), button_row_container_->GetHeightForWidth(width())));
@@ -178,7 +186,7 @@ void DialogClientView::Layout() {
 bool DialogClientView::AcceleratorPressed(const ui::Accelerator& accelerator) {
   DCHECK_EQ(accelerator.key_code(), ui::VKEY_ESCAPE);
 
-  GetWidget()->Close();
+  GetWidget()->CloseWithReason(Widget::ClosedReason::kEscKeyPressed);
   return true;
 }
 
@@ -234,6 +242,9 @@ void DialogClientView::ButtonPressed(Button* sender, const ui::Event& event) {
   if (!GetDialogDelegate())
     return;
 
+  if (IsPossiblyUnintendedInteraction(view_shown_time_stamp_, event))
+    return;
+
   if (sender == ok_button_)
     AcceptWindow();
   else if (sender == cancel_button_)
@@ -242,6 +253,10 @@ void DialogClientView::ButtonPressed(Button* sender, const ui::Event& event) {
     NOTREACHED();
 }
 
+void DialogClientView::ResetViewShownTimeStampForTesting() {
+  view_shown_time_stamp_ = base::TimeTicks();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // DialogClientView, private:
 
@@ -261,7 +276,7 @@ void DialogClientView::ChildVisibilityChanged(View* child) {
   ChildPreferredSizeChanged(child);
 }
 
-void DialogClientView::OnDialogModelChanged() {
+void DialogClientView::OnDialogChanged() {
   UpdateDialogButtons();
 }
 
