@@ -42,11 +42,22 @@
 #include "api/qwebenginemessagepumpscheduler_p.h"
 
 #include "base/message_loop/message_loop.h"
+#include "base/message_loop/message_loop_impl.h"
+#include "base/message_loop/message_loop_current.h"
 #include "base/process/process.h"
 #include "base/threading/thread_restrictions.h"
 #include "content/public/browser/browser_main_parts.h"
 #include "content/public/browser/browser_thread.h"
+#include "content/public/browser/child_process_security_policy.h"
 #include "content/public/common/service_manager_connection.h"
+#include "extensions/buildflags/buildflags.h"
+#if BUILDFLAG(ENABLE_EXTENSIONS)
+#include "extensions/common/constants.h"
+#include "extensions/common/extensions_client.h"
+#include "extensions/extensions_browser_client_qt.h"
+#include "extensions/extension_system_factory_qt.h"
+#include "common/extensions/extensions_client_qt.h"
+#endif //BUILDFLAG(ENABLE_EXTENSIONS)
 #include "services/resource_coordinator/public/cpp/process_resource_coordinator.h"
 #include "services/resource_coordinator/public/cpp/resource_coordinator_features.h"
 #include "services/service_manager/public/cpp/connector.h"
@@ -56,7 +67,12 @@
 #include "service/service_qt.h"
 #include "web_engine_context.h"
 
-#include <QEventLoop>
+#include <QtGui/qtgui-config.h>
+
+#if QT_CONFIG(opengl)
+#include "ui/gl/gl_context.h"
+#include <QOpenGLContext>
+#endif
 
 #if defined(OS_MACOSX)
 #include "ui/base/idle/idle.h"
@@ -89,6 +105,8 @@ int GetTimeIntervalMilliseconds(const base::TimeTicks &from)
     return delay < 0 ? 0 : delay;
 }
 
+}  // anonymous namespace
+
 class MessagePumpForUIQt : public base::MessagePump
 {
 public:
@@ -96,43 +114,86 @@ public:
         : m_scheduler([this]() { handleScheduledWork(); })
     {}
 
+    void setDelegate(Delegate *delegate)
+    {
+        m_delegate = delegate;
+    }
+
     void Run(Delegate *delegate) override
     {
-        if (!m_delegate)
-            m_delegate = delegate;
-        else
-            Q_ASSERT(delegate == m_delegate);
         // This is used only when MessagePumpForUIQt is used outside of the GUI thread.
-        QEventLoop loop;
-        m_explicitLoop = &loop;
-        loop.exec();
-        m_explicitLoop = nullptr;
+        NOTIMPLEMENTED();
     }
 
     void Quit() override
     {
-        Q_ASSERT(m_explicitLoop);
-        m_explicitLoop->quit();
+        // This is used only when MessagePumpForUIQt is used outside of the GUI thread.
+        NOTIMPLEMENTED();
     }
 
     void ScheduleWork() override
     {
         // NOTE: This method may called from any thread at any time.
-        if (!m_delegate)
-            m_delegate = base::MessageLoopForUI::current();
         m_scheduler.scheduleWork();
     }
 
     void ScheduleDelayedWork(const base::TimeTicks &delayed_work_time) override
     {
-        if (!m_delegate)
-            m_delegate = base::MessageLoopForUI::current();
         m_scheduler.scheduleDelayedWork(GetTimeIntervalMilliseconds(delayed_work_time));
     }
 
 private:
+    // Both Qt and Chromium keep track of the current GL context by using
+    // thread-local variables, and naturally they are completely unaware of each
+    // other. As a result, when a QOpenGLContext is made current, the previous
+    // gl::GLContext is not released, and vice-versa. This is fine as long as
+    // each thread uses exclusively either Qt or Chromium GL bindings, which is
+    // usually the case.
+    //
+    // The only exception is when the GL driver is considered thread-unsafe
+    // (QOpenGLContext::supportsThreadedOpenGL() is false), in which case we
+    // have to run all GL operations, including Chromium's GPU service, on the
+    // UI thread. Now the bindings are being mixed and both Qt and Chromium get
+    // quite confused regarding the current state of the surface.
+    //
+    // To get this to work we have to release the current QOpenGLContext before
+    // executing any tasks from Chromium's GPU service and the gl::GLContext
+    // afterwards. Since GPU service just posts tasks to the UI thread task
+    // runner, we'll have to instrument the entire UI thread message pump.
+    class ScopedGLContextChecker
+    {
+#if QT_CONFIG(opengl)
+    public:
+        ScopedGLContextChecker()
+        {
+            if (!m_enabled)
+                return;
+
+            if (QOpenGLContext *context = QOpenGLContext::currentContext())
+                context->doneCurrent();
+        }
+
+        ~ScopedGLContextChecker()
+        {
+            if (!m_enabled)
+                return;
+
+            if (gl::GLContext *context = gl::GLContext::GetCurrent())
+                context->ReleaseCurrent(nullptr);
+        }
+
+    private:
+        bool m_enabled = !QOpenGLContext::supportsThreadedOpenGL();
+#endif // QT_CONFIG(opengl)
+    };
+
+
     void handleScheduledWork()
     {
+        Q_ASSERT(m_delegate);
+
+        ScopedGLContextChecker glContextChecker;
+
         bool more_work_is_plausible = m_delegate->DoWork();
 
         base::TimeTicks delayed_work_time;
@@ -149,16 +210,26 @@ private:
     }
 
     Delegate *m_delegate = nullptr;
-    QEventLoop *m_explicitLoop = nullptr;
     QWebEngineMessagePumpScheduler m_scheduler;
 };
 
-}  // anonymous namespace
+// Needed to access protected constructor from MessageLoop.
+class MessageLoopForUIQt : public base::MessageLoop {
+public:
+    MessageLoopForUIQt() : MessageLoop(TYPE_UI, base::BindOnce(&messagePumpFactory))
+    {
+        BindToCurrentThread();
 
-std::unique_ptr<base::MessagePump> messagePumpFactory()
-{
-    return base::WrapUnique(new MessagePumpForUIQt);
-}
+        auto pump = static_cast<MessagePumpForUIQt *>(pump_);
+        auto backend = static_cast<base::MessageLoopImpl *>(backend_.get());
+        pump->setDelegate(backend);
+    }
+private:
+    static std::unique_ptr<base::MessagePump> messagePumpFactory()
+    {
+        return base::WrapUnique(new MessagePumpForUIQt);
+    }
+};
 
 BrowserMainPartsQt::BrowserMainPartsQt() : content::BrowserMainParts()
 { }
@@ -168,12 +239,25 @@ BrowserMainPartsQt::~BrowserMainPartsQt() = default;
 
 int BrowserMainPartsQt::PreEarlyInitialization()
 {
-    base::MessageLoop::InitMessagePumpForUIFactory(messagePumpFactory);
+#if BUILDFLAG(ENABLE_EXTENSIONS)
+    content::ChildProcessSecurityPolicy::GetInstance()->RegisterWebSafeScheme(extensions::kExtensionScheme);
+#endif //ENABLE_EXTENSIONS
     return 0;
 }
 
 void BrowserMainPartsQt::PreMainMessageLoopStart()
 {
+    // Overrides message loop creation in BrowserMainLoop::MainMessageLoopStart().
+    m_mainMessageLoop.reset(new MessageLoopForUIQt);
+}
+
+void BrowserMainPartsQt::PreMainMessageLoopRun()
+{
+#if BUILDFLAG(ENABLE_EXTENSIONS)
+    extensions::ExtensionsClient::Set(new extensions::ExtensionsClientQt());
+    extensions::ExtensionsBrowserClient::Set(new extensions::ExtensionsBrowserClientQt());
+    extensions::ExtensionSystemFactoryQt::GetInstance();
+#endif //ENABLE_EXTENSIONS
 }
 
 void BrowserMainPartsQt::PostMainMessageLoopRun()
@@ -203,12 +287,9 @@ int BrowserMainPartsQt::PreCreateThreads()
 void BrowserMainPartsQt::ServiceManagerConnectionStarted(content::ServiceManagerConnection *connection)
 {
     ServiceQt::GetInstance()->InitConnector();
-    connection->GetConnector()->StartService(service_manager::Identity("qtwebengine"));
-    if (resource_coordinator::IsResourceCoordinatorEnabled()) {
-        m_processResourceCoordinator = std::make_unique<resource_coordinator::ProcessResourceCoordinator>(connection->GetConnector());
-        m_processResourceCoordinator->SetLaunchTime(base::Time::Now());
-        m_processResourceCoordinator->SetPID(base::Process::Current().Pid());
-    }
+    connection->GetConnector()->WarmService(service_manager::ServiceFilter::ByName("qtwebengine"));
+    m_processResourceCoordinator = std::make_unique<resource_coordinator::ProcessResourceCoordinator>(connection->GetConnector());
+    m_processResourceCoordinator->OnProcessLaunched(base::Process::Current());
 }
 
 } // namespace QtWebEngineCore
