@@ -53,50 +53,43 @@
 
 #include "base/values.h"
 #include "base/memory/ref_counted_memory.h"
-#include "base/task_scheduler/post_task.h"
+#include "base/task/post_task.h"
 #include "chrome/browser/printing/print_job_manager.h"
 #include "chrome/browser/printing/printer_query.h"
 #include "components/printing/common/print_messages.h"
 #include "content/browser/renderer_host/render_view_host_impl.h"
 #include "content/browser/web_contents/web_contents_impl.h"
+#include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/common/web_preferences.h"
-#include "printing/pdf_metafile_skia.h"
+#include "printing/metafile_skia.h"
 #include "printing/print_job_constants.h"
 #include "printing/units.h"
 
-DEFINE_WEB_CONTENTS_USER_DATA_KEY(QtWebEngineCore::PrintViewManagerQt);
-
 namespace {
 
 static const qreal kMicronsToMillimeter = 1000.0f;
 
-static std::vector<char>
-GetStdVectorFromHandle(base::SharedMemoryHandle handle, uint32_t data_size)
+static QSharedPointer<QByteArray> GetStdVectorFromHandle(const base::ReadOnlySharedMemoryRegion &handle)
 {
-    std::unique_ptr<base::SharedMemory> shared_buf(
-                new base::SharedMemory(handle, true));
+    base::ReadOnlySharedMemoryMapping map = handle.Map();
+    if (!map.IsValid())
+        return QSharedPointer<QByteArray>(new QByteArray);
 
-    if (!shared_buf->Map(data_size)) {
-        return std::vector<char>();
-    }
-
-    char* data = static_cast<char*>(shared_buf->memory());
-    return std::vector<char>(data, data + data_size);
+    const char* data = static_cast<const char*>(map.memory());
+    return QSharedPointer<QByteArray>(new QByteArray(data, map.size()));
 }
 
 static scoped_refptr<base::RefCountedBytes>
-GetBytesFromHandle(base::SharedMemoryHandle handle, uint32_t data_size)
+GetBytesFromHandle(const base::ReadOnlySharedMemoryRegion &handle)
 {
-    std::unique_ptr<base::SharedMemory> shared_buf(new base::SharedMemory(handle, true));
-
-    if (!shared_buf->Map(data_size)) {
+    base::ReadOnlySharedMemoryMapping map = handle.Map();
+    if (!map.IsValid())
         return nullptr;
-    }
 
-    unsigned char* data = static_cast<unsigned char*>(shared_buf->memory());
-    std::vector<unsigned char> dataVector(data, data + data_size);
+    const unsigned char* data = static_cast<const unsigned char*>(map.memory());
+    std::vector<unsigned char> dataVector(data, data + map.size());
     return base::RefCountedBytes::TakeVector(&dataVector);
 }
 
@@ -105,18 +98,17 @@ static void SavePdfFile(scoped_refptr<base::RefCountedBytes> data,
                         const base::FilePath &path,
                         const QtWebEngineCore::PrintViewManagerQt::PrintToPDFFileCallback &saveCallback)
 {
-    base::AssertBlockingAllowed();
+    base::AssertBlockingAllowedDeprecated();
     DCHECK_GT(data->size(), 0U);
 
-    printing::PdfMetafileSkia metafile;
+    printing::MetafileSkia metafile;
     metafile.InitFromData(static_cast<const void*>(data->front()), data->size());
 
     base::File file(path,
                     base::File::FLAG_CREATE_ALWAYS | base::File::FLAG_WRITE);
     bool success = file.IsValid() && metafile.SaveTo(&file);
-    content::BrowserThread::PostTask(content::BrowserThread::UI,
-                                     FROM_HERE,
-                                     base::Bind(saveCallback, success));
+    base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                             base::BindOnce(saveCallback, success));
 }
 
 static base::DictionaryValue *createPrintSettings()
@@ -139,14 +131,15 @@ static base::DictionaryValue *createPrintSettings()
 
     printSettings->SetInteger(printing::kSettingDuplexMode, printing::SIMPLEX);
     printSettings->SetInteger(printing::kSettingCopies, 1);
+    printSettings->SetInteger(printing::kSettingPagesPerSheet, 1);
     printSettings->SetBoolean(printing::kSettingCollate, false);
 //    printSettings->SetBoolean(printing::kSettingGenerateDraftData, false);
     printSettings->SetBoolean(printing::kSettingPreviewModifiable, false);
 
-    printSettings->SetBoolean(printing::kSettingShouldPrintSelectionOnly, false);
-    printSettings->SetBoolean(printing::kSettingShouldPrintBackgrounds, true);
-    printSettings->SetBoolean(printing::kSettingHeaderFooterEnabled, false);
-    printSettings->SetBoolean(printing::kSettingRasterizePdf, false);
+    printSettings->SetKey(printing::kSettingShouldPrintSelectionOnly, base::Value(false));
+    printSettings->SetKey(printing::kSettingShouldPrintBackgrounds, base::Value(true));
+    printSettings->SetKey(printing::kSettingHeaderFooterEnabled, base::Value(false));
+    printSettings->SetKey(printing::kSettingRasterizePdf, base::Value(false));
     printSettings->SetInteger(printing::kSettingScaleFactor, 100);
     printSettings->SetString(printing::kSettingDeviceName, "");
     printSettings->SetInteger(printing::kPreviewUIID, 12345678);
@@ -224,16 +217,16 @@ void PrintViewManagerQt::PrintToPDFFileWithCallback(const QPageLayout &pageLayou
         return;
 
     if (m_printSettings || !filePath.length()) {
-        content::BrowserThread::PostTask(content::BrowserThread::UI, FROM_HERE,
-                                         base::Bind(callback, false));
+        base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                                 base::BindOnce(callback, false));
         return;
     }
 
     m_pdfOutputPath = toFilePath(filePath);
     m_pdfSaveCallback = callback;
     if (!PrintToPDFInternal(pageLayout, printInColor)) {
-        content::BrowserThread::PostTask(content::BrowserThread::UI, FROM_HERE,
-                                         base::Bind(callback, false));
+        base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                                 base::BindOnce(callback, false));
         resetPdfState();
     }
 }
@@ -248,16 +241,15 @@ void PrintViewManagerQt::PrintToPDFWithCallback(const QPageLayout &pageLayout,
 
     // If there already is a pending print in progress, don't try starting another one.
     if (m_printSettings) {
-        content::BrowserThread::PostTask(content::BrowserThread::UI, FROM_HERE,
-                                         base::Bind(callback, std::vector<char>()));
+        base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                                base::BindOnce(callback, QSharedPointer<QByteArray>()));
         return;
     }
 
     m_pdfPrintCallback = callback;
     if (!PrintToPDFInternal(pageLayout, printInColor, useCustomMargins)) {
-        content::BrowserThread::PostTask(content::BrowserThread::UI,
-                                         FROM_HERE,
-                                         base::Bind(callback, std::vector<char>()));
+        base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                                 base::BindOnce(callback, QSharedPointer<QByteArray>()));
 
         resetPdfState();
     }
@@ -360,21 +352,18 @@ void PrintViewManagerQt::OnMetafileReadyForPrinting(content::RenderFrameHost* rf
     StopWorker(params.document_cookie);
 
     // Create local copies so we can reset the state and take a new pdf print job.
-    base::Callback<void(const std::vector<char>&)> pdf_print_callback = m_pdfPrintCallback;
-    base::Callback<void(bool)> pdf_save_callback = m_pdfSaveCallback;
+    PrintToPDFCallback pdf_print_callback = std::move(m_pdfPrintCallback);
+    PrintToPDFFileCallback pdf_save_callback = std::move(m_pdfSaveCallback);
     base::FilePath pdfOutputPath = m_pdfOutputPath;
 
     resetPdfState();
 
     if (!pdf_print_callback.is_null()) {
-        std::vector<char> data_vector = GetStdVectorFromHandle(params.content.metafile_data_handle,
-                                                               params.content.data_size);
-        content::BrowserThread::PostTask(content::BrowserThread::UI,
-                                         FROM_HERE,
-                                         base::Bind(pdf_print_callback, data_vector));
+        QSharedPointer<QByteArray> data_array = GetStdVectorFromHandle(params.content.metafile_data_region);
+        base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                                 base::BindOnce(pdf_print_callback, data_array));
     } else {
-        scoped_refptr<base::RefCountedBytes> data_bytes
-                = GetBytesFromHandle(params.content.metafile_data_handle, params.content.data_size);
+        scoped_refptr<base::RefCountedBytes> data_bytes = GetBytesFromHandle(params.content.metafile_data_region);
         base::PostTaskWithTraits(FROM_HERE, {base::MayBlock()},
                                  base::BindOnce(&SavePdfFile, data_bytes, pdfOutputPath, pdf_save_callback));
     }
@@ -394,9 +383,8 @@ void PrintViewManagerQt::DidStartLoading()
 void PrintViewManagerQt::NavigationStopped()
 {
     if (!m_pdfPrintCallback.is_null()) {
-        content::BrowserThread::PostTask(content::BrowserThread::UI,
-                                         FROM_HERE,
-                                         base::Bind(m_pdfPrintCallback, std::vector<char>()));
+        base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                                 base::BindOnce(m_pdfPrintCallback, QSharedPointer<QByteArray>()));
     }
     resetPdfState();
     PrintViewManagerBaseQt::NavigationStopped();
@@ -406,9 +394,8 @@ void PrintViewManagerQt::RenderProcessGone(base::TerminationStatus status)
 {
     PrintViewManagerBaseQt::RenderProcessGone(status);
     if (!m_pdfPrintCallback.is_null()) {
-        content::BrowserThread::PostTask(content::BrowserThread::UI,
-                                         FROM_HERE,
-                                         base::Bind(m_pdfPrintCallback, std::vector<char>()));
+        base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
+                                 base::BindOnce(m_pdfPrintCallback, QSharedPointer<QByteArray>()));
     }
     resetPdfState();
 }
@@ -450,4 +437,6 @@ void PrintViewManagerQt::PrintPreviewDone() {
     m_printPreviewRfh = nullptr;
 }
 
+WEB_CONTENTS_USER_DATA_KEY_IMPL(PrintViewManagerQt)
+
 } // namespace QtWebEngineCore
