@@ -103,7 +103,7 @@ static bool scriptMatchesURL(const UserScriptData &scriptData, const GURL &url)
         matchFound = false;
         for (auto it = scriptData.urlPatterns.begin(), end = scriptData.urlPatterns.end(); it != end; ++it) {
             URLPattern urlPattern(validUserScriptSchemes());
-            if (urlPattern.Parse(*it) == URLPattern::PARSE_SUCCESS && urlPattern.MatchesURL(url))
+            if (urlPattern.Parse(*it) == URLPattern::ParseResult::kSuccess && urlPattern.MatchesURL(url))
                 matchFound = true;
         }
         if (!matchFound)
@@ -137,8 +137,7 @@ public:
 
 private:
     // RenderFrameObserver implementation.
-    void DidCommitProvisionalLoad(bool is_new_navigation, bool is_same_document_navigation) override;
-    void DidClearWindowObject() override;
+    void DidCommitProvisionalLoad(bool is_same_document_navigation, ui::PageTransition transition) override;
     void DidFinishDocumentLoad() override;
     void DidFinishLoad() override;
     void FrameDetached() override;
@@ -230,8 +229,8 @@ UserResourceController::RenderViewObserverHelper::RenderViewObserverHelper(conte
 {
 }
 
-void UserResourceController::RenderFrameObserverHelper::DidCommitProvisionalLoad(bool /* is_new_navigation */,
-                                                                                 bool is_same_document_navigation)
+void UserResourceController::RenderFrameObserverHelper::DidCommitProvisionalLoad(bool is_same_document_navigation,
+                                                                                 ui::PageTransition /*transitionbool*/)
 {
     if (is_same_document_navigation)
         return;
@@ -241,14 +240,10 @@ void UserResourceController::RenderFrameObserverHelper::DidCommitProvisionalLoad
     // that the WebChannelTransportHost is ready to receive messages.
 
     m_runner.reset(new Runner(render_frame()->GetWebFrame()));
-}
 
-void UserResourceController::RenderFrameObserverHelper::DidClearWindowObject()
-{
-    // This is called both before and after DidCommitProvisionalLoad, non-null
-    // m_runner means it's after.
-    if (m_runner)
-        m_runner->run(UserScriptData::DocumentElementCreation);
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE,
+            base::BindOnce(&Runner::run, m_runner->AsWeakPtr(), UserScriptData::DocumentElementCreation));
 }
 
 void UserResourceController::RenderFrameObserverHelper::DidFinishDocumentLoad()
