@@ -91,7 +91,7 @@ class RenderViewObserverHostQt;
 class WebChannelIPCTransportHost;
 class WebEngineContext;
 
-class QWEBENGINECORE_PRIVATE_EXPORT WebContentsAdapter : public QEnableSharedFromThis<WebContentsAdapter> {
+class Q_WEBENGINECORE_PRIVATE_EXPORT WebContentsAdapter : public QEnableSharedFromThis<WebContentsAdapter> {
 public:
     static QSharedPointer<WebContentsAdapter> createFromSerializedNavigationHistory(QDataStream &input, WebContentsAdapterClient *adapterClient);
     WebContentsAdapter();
@@ -183,8 +183,8 @@ public:
     void grantMediaAccessPermission(const QUrl &securityOrigin, WebContentsAdapterClient::MediaRequestFlags flags);
     void runGeolocationRequestCallback(const QUrl &securityOrigin, bool allowed);
     void grantMouseLockPermission(bool granted);
+    void runUserNotificationRequestCallback(const QUrl &securityOrigin, bool allowed);
 
-    void dpiScaleChanged();
     void setBackgroundColor(const QColor &color);
     QAccessibleInterface *browserAccessible();
     ProfileQt* profile();
