@@ -40,7 +40,6 @@
 #ifndef WEB_CONTENTS_DELEGATE_QT_H
 #define WEB_CONTENTS_DELEGATE_QT_H
 
-#include "content/browser/frame_host/frame_tree_node.h"
 #include "content/public/browser/web_contents_delegate.h"
 #include "content/public/browser/web_contents_observer.h"
 #include "third_party/skia/include/core/SkColor.h"
@@ -71,23 +70,6 @@ class WebContentsAdapter;
 class WebContentsAdapterClient;
 class WebEngineSettings;
 
-class FrameFocusedObserver : public content::FrameTreeNode::Observer
-{
-public:
-    FrameFocusedObserver(WebContentsAdapterClient *adapterClient);
-    ~FrameFocusedObserver();
-    void addNode(content::FrameTreeNode *node);
-    void removeNode(content::FrameTreeNode *node);
-
-protected:
-    void OnFrameTreeNodeFocused(content::FrameTreeNode *node) override;
-    void OnFrameTreeNodeDestroyed(content::FrameTreeNode *node) override;
-
-private:
-    WebContentsAdapterClient *m_viewClient;
-    QVector<content::FrameTreeNode *> m_observedNodes;
-};
-
 class SavePageInfo
 {
 public:
@@ -114,7 +96,6 @@ public:
     QString lastSearchedString() const { return m_lastSearchedString; }
     void setLastSearchedString(const QString &s) { m_lastSearchedString = s; }
     int lastReceivedFindReply() const { return m_lastReceivedFindReply; }
-    void setLastReceivedFindReply(int id) { m_lastReceivedFindReply = id; }
 
     QUrl url() const { return m_url; }
     QString title() const { return m_title; }
@@ -125,7 +106,7 @@ public:
     void AddNewContents(content::WebContents *source, std::unique_ptr<content::WebContents> new_contents, WindowOpenDisposition disposition, const gfx::Rect &initial_pos, bool user_gesture, bool *was_blocked) override;
     void CloseContents(content::WebContents *source) override;
     void LoadProgressChanged(content::WebContents* source, double progress) override;
-    void HandleKeyboardEvent(content::WebContents *source, const content::NativeWebKeyboardEvent &event) override;
+    bool HandleKeyboardEvent(content::WebContents *source, const content::NativeWebKeyboardEvent &event) override;
     content::ColorChooser* OpenColorChooser(content::WebContents *source, SkColor color, const std::vector<blink::mojom::ColorSuggestionPtr> &suggestions) override;
     void WebContentsCreated(content::WebContents *source_contents, int opener_render_process_id, int opener_render_frame_id,
                             const std::string &frame_name, const GURL &target_url, content::WebContents *new_contents) override;
@@ -133,7 +114,9 @@ public:
     void EnterFullscreenModeForTab(content::WebContents *web_contents, const GURL &origin, const blink::WebFullscreenOptions &) override;
     void ExitFullscreenModeForTab(content::WebContents*) override;
     bool IsFullscreenForTabOrPending(const content::WebContents* web_contents) const override;
-    void RunFileChooser(content::RenderFrameHost* render_frame_host, const content::FileChooserParams& params) override;
+    void RunFileChooser(content::RenderFrameHost* render_frame_host,
+                        std::unique_ptr<content::FileSelectListener> listener,
+                        const blink::mojom::FileChooserParams& params) override;
     bool DidAddMessageToConsole(content::WebContents* source, int32_t level, const base::string16& message, int32_t line_no, const base::string16& source_id) override;
     void FindReply(content::WebContents *source, int request_id, int number_of_matches, const gfx::Rect& selection_rect, int active_match_ordinal, bool final_update) override;
     void RequestMediaAccessPermission(content::WebContents *web_contents,
@@ -143,21 +126,19 @@ public:
     void UpdateTargetURL(content::WebContents* source, const GURL& url) override;
     void RequestToLockMouse(content::WebContents *web_contents, bool user_gesture, bool last_unlocked_by_target) override;
     void BeforeUnloadFired(content::WebContents* tab, bool proceed, bool* proceed_to_fire_unload) override;
-    bool CheckMediaAccessPermission(content::RenderFrameHost* render_frame_host, const GURL& security_origin, content::MediaStreamType type) override;
+    bool CheckMediaAccessPermission(content::RenderFrameHost* render_frame_host, const GURL& security_origin, blink::MediaStreamType type) override;
     void RegisterProtocolHandler(content::WebContents* web_contents, const std::string& protocol, const GURL& url, bool user_gesture) override;
     void UnregisterProtocolHandler(content::WebContents* web_contents, const std::string& protocol, const GURL& url, bool user_gesture) override;
     bool TakeFocus(content::WebContents *source, bool reverse) override;
 
     // WebContentsObserver overrides
-    void RenderFrameCreated(content::RenderFrameHost *render_frame_host) override;
     void RenderFrameDeleted(content::RenderFrameHost *render_frame_host) override;
-    void RenderFrameHostChanged(content::RenderFrameHost *old_host, content::RenderFrameHost *new_host) override;
     void RenderViewHostChanged(content::RenderViewHost *old_host, content::RenderViewHost *new_host) override;
     void DidStartNavigation(content::NavigationHandle *navigation_handle) override;
     void DidFinishNavigation(content::NavigationHandle *navigation_handle) override;
     void DidFailLoad(content::RenderFrameHost* render_frame_host, const GURL& validated_url, int error_code, const base::string16& error_description) override;
     void DidFinishLoad(content::RenderFrameHost *render_frame_host, const GURL &validated_url) override;
-    void BeforeUnloadFired(const base::TimeTicks& proceed_time) override;
+    void BeforeUnloadFired(bool proceed, const base::TimeTicks& proceed_time) override;
     void DidUpdateFaviconURL(const std::vector<content::FaviconURL> &candidates) override;
     void OnVisibilityChanged(content::Visibility visibility) override;
     void DidFirstVisuallyNonEmptyPaint() override;
@@ -168,6 +149,7 @@ public:
     void allowCertificateError(const QSharedPointer<CertificateErrorController> &);
     void selectClientCert(const QSharedPointer<ClientCertSelectController> &);
     void requestGeolocationPermission(const QUrl &requestingOrigin);
+    void requestUserNotificationPermission(const QUrl &requestingOrigin);
     void launchExternalURL(const QUrl &url, ui::PageTransition page_transition, bool is_main_frame, bool has_user_gesture);
     FaviconManager *faviconManager();
 
@@ -193,7 +175,6 @@ private:
     QSharedPointer<FilePickerController> m_filePickerController;
     QUrl m_initialTargetUrl;
     int m_lastLoadProgress;
-    FrameFocusedObserver m_frameFocusedObserver;
 
     QUrl m_url;
     QString m_title;
