@@ -46,6 +46,7 @@
 #include "qquickwebenginesettings_p.h"
 #include "qquickwebengineview_p_p.h"
 #include "qwebenginecookiestore.h"
+#include "qwebenginenotification.h"
 
 #include <QQmlEngine>
 
@@ -150,6 +151,13 @@ ASSERT_ENUMS_MATCH(QQuickWebEngineDownloadItem::MimeHtmlSaveFormat, QtWebEngineC
   The \a download argument holds the state of the finished download instance.
 */
 
+/*!
+    \fn QQuickWebEngineProfile::presentNotification(QWebEngineNotification *notification)
+
+    This signal is emitted whenever there is a newly created user notification.
+    The \a notification argument holds the notification instance to query data and interact with.
+*/
+
 QQuickWebEngineProfilePrivate::QQuickWebEngineProfilePrivate(ProfileAdapter *profileAdapter)
         : m_settings(new QQuickWebEngineSettings())
         , m_profileAdapter(profileAdapter)
@@ -285,6 +293,20 @@ void QQuickWebEngineProfilePrivate::downloadUpdated(const DownloadItemInfo &info
     }
 }
 
+void QQuickWebEngineProfilePrivate::useForGlobalCertificateVerificationChanged()
+{
+    Q_Q(QQuickWebEngineProfile);
+    Q_EMIT q->useForGlobalCertificateVerificationChanged();
+}
+
+void QQuickWebEngineProfilePrivate::showNotification(QSharedPointer<QtWebEngineCore::UserNotificationController> &controller)
+{
+    Q_Q(QQuickWebEngineProfile);
+    auto notification = new QWebEngineNotification(controller);
+    QQmlEngine::setObjectOwnership(notification, QQmlEngine::JavaScriptOwnership);
+    Q_EMIT q->presentNotification(notification);
+}
+
 void QQuickWebEngineProfilePrivate::userScripts_append(QQmlListProperty<QQuickWebEngineScript> *p, QQuickWebEngineScript *script)
 {
     Q_ASSERT(p && p->data);
@@ -364,6 +386,14 @@ void QQuickWebEngineProfilePrivate::userScripts_clear(QQmlListProperty<QQuickWeb
     The \a download argument holds the state of the finished download instance.
 */
 
+/*!
+    \qmlsignal WebEngineProfile::presentNotification(WebEngineNotification notification)
+    \since QtWebEngine 1.9
+
+    This signal is emitted whenever there is a newly created user notification.
+    The \a notification argument holds the notification instance to query data and interact with.
+*/
+
 /*!
     Constructs a new profile with the parent \a parent.
 */
@@ -795,6 +825,102 @@ bool QQuickWebEngineProfile::isSpellCheckEnabled() const
      return d->profileAdapter()->isSpellCheckEnabled();
 }
 
+/*!
+    \property QQuickWebEngineProfile::useForGlobalCertificateVerification
+    \since 5.13
+
+    This property holds whether this profile is used for downloading and
+    caching during global certificate verification when using the online
+    certificate status protocol (OCSP), certificate revokation lists (CRLs),
+    and authority information access (AIA), for example.
+
+    As long as one profile has this option enabled, all other profiles will be
+    able to use it for certificate verification. Only one profile at a time can
+    have this option enabled. It is recommended that the profile has a disk HTTP
+    cache to avoid needlessly re-downloading.
+
+    By default, no profile has this property enabled.
+
+    Currently, only affects Linux/NSS installations, where having a profile with
+    this role enables OCSP.
+*/
+
+/*!
+    \qmlproperty bool WebEngineProfile::useForGlobalCertificateVerification
+    \since QtWebEngine 1.9
+
+    This property holds whether this profile is used for downloading and
+    caching during global certificate verification when using the online
+    certificate status protocol (OCSP), certificate revokation lists (CRLs),
+    and authority information access (AIA), for example.
+
+    As long as one profile has this option enabled, all other profiles will be
+    able to use it for certificate verification. Only one profile at a time can
+    have this option enabled. It is recommended that the profile has a disk HTTP
+    cache to avoid needlessly re-downloading.
+
+    By default, no profile has this property enabled.
+
+    Currently, only affects Linux/NSS installations, where having a profile with
+    this role enables OCSP.
+*/
+
+void QQuickWebEngineProfile::setUseForGlobalCertificateVerification(bool enable)
+{
+    Q_D(QQuickWebEngineProfile);
+    if (enable != d->profileAdapter()->isUsedForGlobalCertificateVerification()) {
+        d->profileAdapter()->setUseForGlobalCertificateVerification(enable);
+        emit useForGlobalCertificateVerificationChanged();
+    }
+}
+
+bool QQuickWebEngineProfile::isUsedForGlobalCertificateVerification() const
+{
+     const Q_D(QQuickWebEngineProfile);
+     return d->profileAdapter()->isUsedForGlobalCertificateVerification();
+}
+
+/*!
+    \qmlproperty string WebEngineProfile::downloadPath
+    \since  QtWebEngine 1.9
+
+    The path to the location where the downloaded files are stored.
+
+    Overrides the default path used for download location.
+
+    If set to an empty string, the default path is restored.
+
+    \note By default, the download path is QStandardPaths::DownloadLocation.
+*/
+
+/*!
+    \property QQuickWebEngineProfile::downloadPath
+    \since QtWebEngine 1.9
+
+    The path to the location where the downloaded files are stored.
+
+    Overrides the default path used for download location, setting it to \a path.
+
+    If set to an empty string, the default path is restored.
+
+    \note By default, the download path is QStandardPaths::DownloadLocation.
+*/
+
+void QQuickWebEngineProfile::setDownloadPath(const QString &path)
+{
+    Q_D(QQuickWebEngineProfile);
+    if (downloadPath() == path)
+        return;
+    d->profileAdapter()->setDownloadPath(path);
+    emit downloadPathChanged();
+}
+
+QString QQuickWebEngineProfile::downloadPath() const
+{
+    const Q_D(QQuickWebEngineProfile);
+    return d->profileAdapter()->downloadPath();
+}
+
 /*!
 
     Returns the cookie store for this profile.
@@ -827,40 +953,52 @@ void QQuickWebEngineProfile::clearHttpCache()
     d->profileAdapter()->clearHttpCache();
 }
 
+#if QT_DEPRECATED_SINCE(5, 13)
+/*!
+    Registers a request interceptor singleton \a interceptor to intercept URL requests.
+
+    The profile does not take ownership of the pointer.
+
+    \obsolete
+
+    Interceptors installed with this method will call
+    QWebEngineUrlRequestInterceptor::interceptRequest on the I/O thread. Therefore
+    the user has to provide thread-safe interaction with the other user classes.
+    Use setUrlRequestInterceptor instead.
+
+    \sa QWebEngineUrlRequestInterceptor
+
+*/
+void QQuickWebEngineProfile::setRequestInterceptor(QWebEngineUrlRequestInterceptor *interceptor)
+{
+    Q_D(QQuickWebEngineProfile);
+    interceptor->setProperty("deprecated", true);
+    d->profileAdapter()->setRequestInterceptor(interceptor);
+    qWarning("Use of deprecated not tread-safe setter, use setUrlRequestInterceptor instead.");
+}
+#endif
 
 /*!
     Registers a request interceptor singleton \a interceptor to intercept URL requests.
 
     The profile does not take ownership of the pointer.
 
-    \sa QWebEngineUrlRequestInterceptor
+    \sa QWebEngineUrlRequestInfo QWebEngineUrlRequestInterceptor
 */
-void QQuickWebEngineProfile::setRequestInterceptor(QWebEngineUrlRequestInterceptor *interceptor)
+void QQuickWebEngineProfile::setUrlRequestInterceptor(QWebEngineUrlRequestInterceptor *interceptor)
 {
     Q_D(QQuickWebEngineProfile);
     d->profileAdapter()->setRequestInterceptor(interceptor);
 }
 
+
 /*!
     Returns the custom URL scheme handler register for the URL scheme \a scheme.
 */
 const QWebEngineUrlSchemeHandler *QQuickWebEngineProfile::urlSchemeHandler(const QByteArray &scheme) const
 {
     const Q_D(QQuickWebEngineProfile);
-    if (d->profileAdapter()->customUrlSchemeHandlers().contains(scheme))
-        return d->profileAdapter()->customUrlSchemeHandlers().value(scheme);
-    return 0;
-}
-
-static bool checkInternalScheme(const QByteArray &scheme)
-{
-    static QSet<QByteArray> internalSchemes;
-    if (internalSchemes.isEmpty()) {
-        internalSchemes << QByteArrayLiteral("qrc") << QByteArrayLiteral("data") << QByteArrayLiteral("blob")
-                        << QByteArrayLiteral("http") << QByteArrayLiteral("https") << QByteArrayLiteral("ftp")
-                        << QByteArrayLiteral("javascript");
-    }
-    return internalSchemes.contains(scheme);
+    return d->profileAdapter()->urlSchemeHandler(scheme);
 }
 
 /*!
@@ -872,25 +1010,7 @@ static bool checkInternalScheme(const QByteArray &scheme)
 void QQuickWebEngineProfile::installUrlSchemeHandler(const QByteArray &scheme, QWebEngineUrlSchemeHandler *handler)
 {
     Q_D(QQuickWebEngineProfile);
-    Q_ASSERT(handler);
-    QByteArray canonicalScheme = scheme.toLower();
-    if (checkInternalScheme(canonicalScheme)) {
-        qWarning("Cannot install a URL scheme handler overriding internal scheme: %s", scheme.constData());
-        return;
-    }
-
-    if (d->profileAdapter()->customUrlSchemeHandlers().contains(canonicalScheme)) {
-        if (d->profileAdapter()->customUrlSchemeHandlers().value(canonicalScheme) != handler)
-            qWarning("URL scheme handler already installed for the scheme: %s", scheme.constData());
-        return;
-    }
-
-    if (QWebEngineUrlScheme::schemeByName(canonicalScheme) == QWebEngineUrlScheme())
-        qWarning("Please register the custom scheme '%s' via QWebEngineUrlScheme::registerScheme() "
-                 "before installing the custom scheme handler.", scheme.constData());
-
-    d->profileAdapter()->addCustomUrlSchemeHandler(canonicalScheme, handler);
-    connect(handler, SIGNAL(_q_destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)), this, SLOT(destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)));
+    d->profileAdapter()->installUrlSchemeHandler(scheme, handler);
 }
 
 /*!
@@ -901,10 +1021,7 @@ void QQuickWebEngineProfile::installUrlSchemeHandler(const QByteArray &scheme, Q
 void QQuickWebEngineProfile::removeUrlSchemeHandler(QWebEngineUrlSchemeHandler *handler)
 {
     Q_D(QQuickWebEngineProfile);
-    Q_ASSERT(handler);
-    if (!d->profileAdapter()->removeCustomUrlSchemeHandler(handler))
-        return;
-    disconnect(handler, SIGNAL(_q_destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)), this, SLOT(destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)));
+    d->profileAdapter()->removeUrlSchemeHandler(handler);
 }
 
 /*!
@@ -915,10 +1032,7 @@ void QQuickWebEngineProfile::removeUrlSchemeHandler(QWebEngineUrlSchemeHandler *
 void QQuickWebEngineProfile::removeUrlScheme(const QByteArray &scheme)
 {
     Q_D(QQuickWebEngineProfile);
-    QWebEngineUrlSchemeHandler *handler = d->profileAdapter()->takeCustomUrlSchemeHandler(scheme);
-    if (!handler)
-        return;
-    disconnect(handler, SIGNAL(_q_destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)), this, SLOT(destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)));
+    d->profileAdapter()->removeUrlScheme(scheme);
 }
 
 /*!
@@ -927,12 +1041,7 @@ void QQuickWebEngineProfile::removeUrlScheme(const QByteArray &scheme)
 void QQuickWebEngineProfile::removeAllUrlSchemeHandlers()
 {
     Q_D(QQuickWebEngineProfile);
-    d->profileAdapter()->clearCustomUrlSchemeHandlers();
-}
-
-void QQuickWebEngineProfile::destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler *obj)
-{
-    removeUrlSchemeHandler(obj);
+    d->profileAdapter()->removeAllUrlSchemeHandlers();
 }
 
 QQuickWebEngineSettings *QQuickWebEngineProfile::settings() const
@@ -970,4 +1079,19 @@ QQmlListProperty<QQuickWebEngineScript> QQuickWebEngineProfile::userScripts()
                                                    d->userScripts_clear);
 }
 
+/*!
+    \since 5.13
+
+    Returns the profile's client certificate store.
+*/
+QWebEngineClientCertificateStore *QQuickWebEngineProfile::clientCertificateStore()
+{
+#if QT_CONFIG(ssl)
+    Q_D(QQuickWebEngineProfile);
+    return d->profileAdapter()->clientCertificateStore();
+#else
+    return nullptr;
+#endif
+}
+
 QT_END_NAMESPACE
