@@ -59,8 +59,7 @@ public:
     ~RenderWidgetHostViewQtDelegateQuick();
 
     void initAsPopup(const QRect&) override;
-    QRectF screenRect() const override;
-    QRectF contentsRect() const override;
+    QRectF viewGeometry() const override;
     void setKeyboardFocus() override;
     bool hasKeyboardFocus() override;
     void lockMouse() override;
@@ -82,6 +81,7 @@ public:
     void setInputMethodHints(Qt::InputMethodHints) override { }
     // The QtQuick view doesn't have a backbuffer of its own and doesn't need this
     void setClearColor(const QColor &) override { }
+    bool copySurface(const QRect &rect, const QSize &size, QImage &image) override;
 
 protected:
     bool event(QEvent *event) override;
@@ -112,8 +112,6 @@ private:
     RenderWidgetHostViewQtDelegateClient *m_client;
     QList<QMetaObject::Connection> m_windowConnections;
     bool m_isPopup;
-    bool m_isPasswordInput;
-    QPoint m_lastGlobalPos;
     QQuickWebEngineView *m_view = nullptr;
 };
 
