@@ -29,9 +29,9 @@
 #include "../util.h"
 #include <QtCore/qbuffer.h>
 #include <QtTest/QtTest>
+#include <QtWebEngineCore/qwebengineurlrequestinterceptor.h>
 #include <QtWebEngineCore/qwebengineurlrequestjob.h>
 #include <QtWebEngineCore/qwebenginecookiestore.h>
-#include <QtWebEngineCore/qwebengineurlscheme.h>
 #include <QtWebEngineCore/qwebengineurlschemehandler.h>
 #include <QtWebEngineWidgets/qwebengineprofile.h>
 #include <QtWebEngineWidgets/qwebenginepage.h>
@@ -44,7 +44,6 @@ class tst_QWebEngineProfile : public QObject
     Q_OBJECT
 
 private Q_SLOTS:
-    void initTestCase();
     void init();
     void cleanup();
     void privateProfile();
@@ -55,33 +54,15 @@ private Q_SLOTS:
     void urlSchemeHandlerFailRequest();
     void urlSchemeHandlerFailOnRead();
     void urlSchemeHandlerStreaming();
-    void urlSchemeHandlerInstallation();
+    void urlSchemeHandlerRequestHeaders();
     void customUserAgent();
     void httpAcceptLanguage();
     void downloadItem();
     void changePersistentPath();
     void initiator();
-    void badDeleteOrder();
-    void qtbug_72299(); // this should be the last test
+    void qtbug_71895(); // this should be the last test
 };
 
-void tst_QWebEngineProfile::initTestCase()
-{
-    QWebEngineUrlScheme foo("foo");
-    QWebEngineUrlScheme stream("stream");
-    QWebEngineUrlScheme letterto("letterto");
-    QWebEngineUrlScheme aviancarrier("aviancarrier");
-    foo.setSyntax(QWebEngineUrlScheme::Syntax::Host);
-    stream.setSyntax(QWebEngineUrlScheme::Syntax::HostAndPort);
-    stream.setDefaultPort(8080);
-    letterto.setSyntax(QWebEngineUrlScheme::Syntax::Path);
-    aviancarrier.setSyntax(QWebEngineUrlScheme::Syntax::Path);
-    QWebEngineUrlScheme::registerScheme(foo);
-    QWebEngineUrlScheme::registerScheme(stream);
-    QWebEngineUrlScheme::registerScheme(letterto);
-    QWebEngineUrlScheme::registerScheme(aviancarrier);
-}
-
 void tst_QWebEngineProfile::init()
 {
     //make sure defualt global profile is 'default' across all the tests
@@ -103,7 +84,6 @@ void tst_QWebEngineProfile::cleanup()
     profile->setCachePath(QString());
     profile->setPersistentStoragePath(QString());
     profile->setHttpCacheType(QWebEngineProfile::DiskHttpCache);
-    profile->removeAllUrlSchemeHandlers();
 }
 
 void tst_QWebEngineProfile::privateProfile()
@@ -393,17 +373,17 @@ public:
     {
     }
 
-    qint64 readData(char *, qint64) Q_DECL_OVERRIDE
+    qint64 readData(char *, qint64) override
     {
         m_job->fail(QWebEngineUrlRequestJob::RequestFailed);
         return -1;
     }
-    qint64 writeData(const char *, qint64) Q_DECL_OVERRIDE
+    qint64 writeData(const char *, qint64) override
     {
         m_job->fail(QWebEngineUrlRequestJob::RequestFailed);
         return -1;
     }
-    void close() Q_DECL_OVERRIDE
+    void close() override
     {
         QIODevice::close();
         deleteLater();
@@ -467,46 +447,65 @@ void tst_QWebEngineProfile::urlSchemeHandlerStreaming()
     QCOMPARE(toPlainTextSync(view.page()), QString::fromLatin1(result));
 }
 
-void tst_QWebEngineProfile::urlSchemeHandlerInstallation()
+class ExtraHeaderInterceptor : public QWebEngineUrlRequestInterceptor
 {
-    FailingUrlSchemeHandler handler;
+public:
+    ExtraHeaderInterceptor() { }
+
+    void setExtraHeader(const QByteArray &key, const QByteArray &value)
+    {
+        m_extraKey = key;
+        m_extraValue = value;
+    }
+
+    void interceptRequest(QWebEngineUrlRequestInfo &info) override
+    {
+        if (info.requestUrl().scheme() == QLatin1String("myscheme"))
+            info.setHttpHeader(m_extraKey, m_extraValue);
+    }
+
+    QByteArray m_extraKey;
+    QByteArray m_extraValue;
+};
+
+class RequestHeadersUrlSchemeHandler : public ReplyingUrlSchemeHandler
+{
+public:
+    void setExpectedHeader(const QByteArray &key, const QByteArray &value)
+    {
+        m_expectedKey = key;
+        m_expectedValue = value;
+    }
+    void requestStarted(QWebEngineUrlRequestJob *job) override
+    {
+        const auto requestHeaders = job->requestHeaders();
+        QVERIFY(requestHeaders.contains(m_expectedKey));
+        QCOMPARE(requestHeaders.value(m_expectedKey), m_expectedValue);
+        ReplyingUrlSchemeHandler::requestStarted(job);
+    }
+    QByteArray m_expectedKey;
+    QByteArray m_expectedValue;
+};
+
+void tst_QWebEngineProfile::urlSchemeHandlerRequestHeaders()
+{
+    RequestHeadersUrlSchemeHandler handler;
+    ExtraHeaderInterceptor interceptor;
+
+    handler.setExpectedHeader("Hello", "World");
+    interceptor.setExtraHeader("Hello", "World");
+
     QWebEngineProfile profile;
+    profile.installUrlSchemeHandler("myscheme", &handler);
+    profile.setRequestInterceptor(&interceptor);
 
-    // Builtin schemes that *cannot* be overridden.
-    for (auto scheme : { "about", "blob", "data", "javascript", "qrc", "https", "http", "file",
-                         "ftp", "wss", "ws", "filesystem", "FileSystem" }) {
-        QTest::ignoreMessage(
-                QtWarningMsg,
-                QRegularExpression("Cannot install a URL scheme handler overriding internal scheme.*"));
-        profile.installUrlSchemeHandler(scheme, &handler);
-        QCOMPARE(profile.urlSchemeHandler(scheme), nullptr);
-    }
-
-    // Builtin schemes that *can* be overridden.
-    for (auto scheme : { "gopher", "GOPHER" }) {
-        profile.installUrlSchemeHandler(scheme, &handler);
-        QCOMPARE(profile.urlSchemeHandler(scheme), &handler);
-        profile.removeUrlScheme(scheme);
-    }
-
-    // Other schemes should be registered with QWebEngineUrlScheme first, but
-    // handler installation still succeeds to preserve backwards compatibility.
-    QTest::ignoreMessage(
-            QtWarningMsg,
-            QRegularExpression("Please register the custom scheme.*"));
-    profile.installUrlSchemeHandler("tst", &handler);
-    QCOMPARE(profile.urlSchemeHandler("tst"), &handler);
-
-    // Existing handler cannot be overridden.
-    FailingUrlSchemeHandler handler2;
-    QTest::ignoreMessage(
-            QtWarningMsg,
-            QRegularExpression("URL scheme handler already installed.*"));
-    profile.installUrlSchemeHandler("tst", &handler2);
-    QCOMPARE(profile.urlSchemeHandler("tst"), &handler);
-    profile.removeUrlScheme("tst");
+    QWebEnginePage page(&profile);
+    QSignalSpy loadFinishedSpy(&page, SIGNAL(loadFinished(bool)));
+    page.load(QUrl(QStringLiteral("myscheme://whatever")));
+    QVERIFY(loadFinishedSpy.wait());
 }
 
+
 void tst_QWebEngineProfile::customUserAgent()
 {
     QString defaultUserAgent = QWebEngineProfile::defaultProfile()->httpUserAgent();
@@ -631,25 +630,7 @@ void tst_QWebEngineProfile::initiator()
     QCOMPARE(handler.initiator, QUrl());
 }
 
-void tst_QWebEngineProfile::badDeleteOrder()
-{
-    QWebEngineProfile *profile = new QWebEngineProfile();
-    QWebEngineView *view = new QWebEngineView();
-    view->resize(640, 480);
-    view->show();
-    QVERIFY(QTest::qWaitForWindowExposed(view));
-    QWebEnginePage *page = new QWebEnginePage(profile, view);
-    view->setPage(page);
-
-    QSignalSpy spyLoadFinished(page, SIGNAL(loadFinished(bool)));
-    page->setHtml(QStringLiteral("<html><body><h1>Badly handled page!</h1></body></html>"));
-    QTRY_COMPARE(spyLoadFinished.count(), 1);
-
-    delete profile;
-    delete view;
-}
-
-void tst_QWebEngineProfile::qtbug_72299()
+void tst_QWebEngineProfile::qtbug_71895()
 {
     QWebEngineView view;
     QSignalSpy loadSpy(view.page(), SIGNAL(loadFinished(bool)));
@@ -659,8 +640,9 @@ void tst_QWebEngineProfile::qtbug_72299()
     view.page()->profile()->setHttpCacheType(QWebEngineProfile::NoCache);
     view.page()->profile()->cookieStore()->deleteAllCookies();
     view.page()->profile()->setPersistentCookiesPolicy(QWebEngineProfile::NoPersistentCookies);
-    QTRY_COMPARE_WITH_TIMEOUT(loadSpy.count(), 1, 20000);
-    QVERIFY(loadSpy.front().front().toBool());
+    bool gotSignal = loadSpy.count() || loadSpy.wait(20000);
+    if (!gotSignal)
+        QSKIP("Couldn't load page from network, skipping test.");
 }
 
 
